/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012-2013 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music;

import oauth.signpost.signature.OAuthMessageSigner;
import oauth.signpost.signature.PlainTextMessageSigner;

import org.apache.http.client.HttpClient;

import android.app.Application;
import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;

import com.ubuntuone.android.music.service.MusicService;
import com.ubuntuone.android.music.util.AccountUtils;
import com.ubuntuone.android.music.util.HttpManager;
import com.ubuntuone.android.music.util.Log;
import com.ubuntuone.android.music.util.NotAuthenticatedException;
import com.ubuntuone.android.music.util.StorageUtils;
import com.ubuntuone.api.music.U1MusicAPI;
import com.ubuntuone.api.sso.authorizer.OAuthAuthorizer;

public class UbuntuOneMusic extends Application
{
	public static final String TAG = UbuntuOneMusic.class.getSimpleName();
	
	/** Should always be true. False would indicate a debug build. */
	public static final boolean IS_RELEASE = !BuildConfig.DEBUG;
	
	// Following flags have been introduced to ease proper test suite runs and debugging.
	
	/** Debug flag indicating if the app should attempt metadata sync. */
	public static final boolean SHOULD_SYNC = IS_RELEASE;
	/** Debug flag indicating if the app should bind to {@link MusicService}. */
	public static final boolean SHOULD_BIND = IS_RELEASE;
	/** Debug flag indicating if the app should actually playback music. */
	public static final boolean SHOULD_PLAY = IS_RELEASE;
	
	// Set this to true to enable foreign key constraint enforcing.
	public static final boolean SHOULD_USE_FK = false;
	
	private static UbuntuOneMusic instance;
	private U1MusicAPI mApi;

	@Override
	public void onCreate() {
		super.onCreate();
		instance = this;
		setupLogging();
		
		StorageUtils.setupMusicDirectory(this);
	}

	private void setupLogging() {
		Log.setLogLevel(android.util.Log.INFO);
		Log.enableCollectingLogs(); // XXX Remove before release.
	}

	public static UbuntuOneMusic getInstance() {
		return instance;
	}

	public static Context getContext() {
		return instance.getApplicationContext();
	}

	public String getApplicationVersion() {
		final Context context = instance;
		String version = null;
		PackageManager pm = context.getPackageManager();
		try {
			PackageInfo info = pm.getPackageInfo(context.getPackageName(), 0);
			version = info.versionName;
		} catch (NameNotFoundException e) {
			// This should never happen as we're looking for our own package.
			Log.e(TAG, "Could not find self package info.", e);
			version = "unknown";
		}
		return version;
	}

	public U1MusicAPI getMusicApi() throws NotAuthenticatedException {
		if (!AccountUtils.isAuthenticated(this)) {
			throw new NotAuthenticatedException();
		}
		
		if (mApi == null) {
			String userAgent = String.format("%s/%s", getPackageName(), getApplicationVersion());
			HttpClient httpClient = HttpManager.getClient();

			String authToken = AccountUtils.getAuthToken(this);
			OAuthMessageSigner signer = new PlainTextMessageSigner();
			OAuthAuthorizer authorizer = OAuthAuthorizer.getWithTokens(authToken, signer);

			mApi = new U1MusicAPI(userAgent, httpClient, authorizer);
			
			setupLogging();
		}
		return mApi;
	}
}
