/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.adapter;

import android.content.Context;
import android.database.Cursor;
import android.support.v4.widget.CursorAdapter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ubuntuone.android.music.adapter.holder.AlbumViewHolder;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.util.U1ImageDownloader;
import com.ubuntuone.android.music.R;

public class U1AlbumAdapter extends CursorAdapter
{
	private Context mContext;
	private LayoutInflater mInflater;
	
	private U1ImageDownloader mImageDownloader = U1ImageDownloader.getAlbumArtDownloader();

	public U1AlbumAdapter(Context context, Cursor c) {
		super(context, c, true);
		mContext = context;
		mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	}
	
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		View view;
		AlbumViewHolder holder;
		if (convertView == null) {
			view = mInflater.inflate(R.layout.list_item_album, null);
			holder = new AlbumViewHolder(view);
			view.setTag(holder);
		} else {
			view = convertView;
			holder = (AlbumViewHolder) convertView.getTag();
		}

		Cursor cursor = getCursor();
		cursor.moveToPosition(position);
		bindView(view, mContext, cursor);
		return view;
	}
	
	@Override
	public void bindView(View view, Context context, Cursor cursor) {
		AlbumViewHolder holder = (AlbumViewHolder) view.getTag();
		
		String albumId = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ID));
		boolean isStarred = cursor.getInt(cursor.getColumnIndex(Albums.STARRED)) != 0;
		String albumTitle = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_TITLE));
		String artist = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ARTIST));
		Integer year = cursor.getInt(cursor.getColumnIndex(Albums.ALBUM_YEAR));
		holder.albumTitle.setText(albumTitle);
		if (year != null && year > 0) {
			holder.albumTitle.setText(String.format("%s%s - %d",
					isStarred ? "✰ " : "", albumTitle, year));
		} else {
			holder.albumTitle.setText(albumTitle);
		}
		holder.albumArtist.setText(artist);
		
		mImageDownloader.getArt(albumId, holder.albumArt);
	}
	
	@Override
	public View newView(Context context, Cursor cursor, ViewGroup parent) {
		// Nothing to do.
		return null;
	}
}
