/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.adapter;

import android.content.Context;
import android.database.Cursor;
import android.support.v4.widget.CursorAdapter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.holder.ArtistViewHolder;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.util.U1ImageDownloader;

public class U1ArtistAdapter extends CursorAdapter
{
	private Context mContext;
	private LayoutInflater mInflater;
	
	private U1ImageDownloader mImageDownloader = U1ImageDownloader.getArtistArtDownloader();

	public U1ArtistAdapter(Context context, Cursor c) {
		super(context, c, true);
		mContext = context;
		mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	}
	
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		View view;
		ArtistViewHolder holder;
		if (convertView == null) {
			view = mInflater.inflate(R.layout.list_item_artist, null);
			holder = new ArtistViewHolder(view);
			view.setTag(holder);
		} else {
			view = convertView;
			holder = (ArtistViewHolder) convertView.getTag();
		}

		Cursor cursor = getCursor();
		cursor.moveToPosition(position);
		bindView(view, mContext, cursor);
		return view;
	}
	
	@Override
	public void bindView(View view, Context context, Cursor cursor) {
		ArtistViewHolder holder = (ArtistViewHolder) view.getTag();
		
		String artistId = cursor.getString(cursor.getColumnIndex(Artists.ARTIST_ID));
		String artistName = cursor.getString(cursor.getColumnIndex(Artists.ARTIST_NAME));
		holder.artistName.setText(artistName);
		
		mImageDownloader.getArt(artistId, holder.artistArt);
	}
	
	@Override
	public View newView(Context context, Cursor cursor, ViewGroup parent) {
		// Nothing to do.
		return null;
	}
}
