/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.adapter;

import android.content.Context;
import android.database.Cursor;
import android.support.v4.widget.CursorAdapter;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.holder.SongViewHolder;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.util.U1ImageDownloader;
import com.ubuntuone.android.music.util.UIUtils;

public class U1SongAdapter extends CursorAdapter
{
	private Context mContext;
	private LayoutInflater mInflater;
	
	private U1ImageDownloader mImageDownloader = U1ImageDownloader.getAlbumArtDownloader();
	
	private boolean mIndicatePlaying;
	private String mAlbumId;
	
	public U1SongAdapter(Context context, Cursor c) {
		this(context, null, false, c, true);
	}
	
	public U1SongAdapter(Context context, String albumId, boolean indicatePlaying,
			Cursor c, boolean autoRequery) {
		super(context, c, autoRequery);
		this.mContext = context;
		this.mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		this.mIndicatePlaying = indicatePlaying;
		this.mAlbumId = albumId;
	}
	
	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		View view;
		SongViewHolder holder;
		if (convertView == null) {
			view = mInflater.inflate(R.layout.list_item_song, null);
			holder = new SongViewHolder(view);
			view.setTag(holder);
		} else {
			view = convertView;
			holder = (SongViewHolder) convertView.getTag();
		}

		Cursor cursor = getCursor();
		cursor.moveToPosition(position);
		bindView(view, mContext, cursor);
		return view;
	}

	@Override
	public void bindView(View view, Context context, Cursor cursor) {
		SongViewHolder holder = (SongViewHolder) view.getTag();
		
		String title = cursor.getString(cursor.getColumnIndex(Songs.SONG_TITLE));
		boolean isStarred = cursor.getInt(cursor.getColumnIndex(Songs.STARRED)) != 0;
		String artist = cursor.getString(cursor.getColumnIndex(Songs.SONG_ARTIST));
		String album = cursor.getString(cursor.getColumnIndex(Songs.SONG_ALBUM));
		String albumId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ALBUM_ID));
		int duration = cursor.getInt(cursor.getColumnIndex(Songs.SONG_DURATION));
		boolean isPlaying = cursor.getInt(cursor.getColumnIndex(Songs.SONG_IS_PLAYING)) != 0;
		boolean isDownloaded = !TextUtils.isEmpty(
				cursor.getString(cursor.getColumnIndex(Songs.SONG_PATH)));

		holder.songTitle.setText(String.format("%s%s%s",
				isDownloaded ? "✔ " : "", isStarred ? "✰ " : "", title));
		if (mAlbumId == null) {
			holder.songArtist.setText(String.format("%s - %s", artist, album));
		} else {
			holder.songArtist.setText(UIUtils.getDuration(duration));
		}
		
		boolean showIsPlaying = mIndicatePlaying && isPlaying;
		holder.songPlaying.setVisibility(showIsPlaying ? View.VISIBLE : View.INVISIBLE);
		
		mImageDownloader.getArt(albumId, holder.songArt);
	}
	
	@Override
	public View newView(Context context, Cursor cursor, ViewGroup parent) {
		// Nothing to do.
		return null;
	}
}
