/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.adapter.holder;

import android.view.View;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.util.U1ImageDownloader;
import com.ubuntuone.android.music.util.UIUtils;

public class PlayerViewHolder
{
	public ImageView albumArt;
	public TextView songTitle;
	public TextView albumTitle;
	public TextView currentTime;
	public TextView totalTime;
	public SeekBar progress;
	
	public PlayerViewHolder(View contentView) {
		albumArt = (ImageView) contentView.findViewById(R.id.album_art);
		songTitle = (TextView) contentView.findViewById(R.id.song_title);
		albumTitle = (TextView) contentView.findViewById(R.id.album_title);
		currentTime = (TextView) contentView.findViewById(R.id.current_time);
		totalTime = (TextView) contentView.findViewById(R.id.total_time);
		progress = (SeekBar) contentView.findViewById(R.id.song_progress);
	}
	
	public void bindView(String songTitle, String albumTitle, String albumId,
			U1ImageDownloader imageDownloader) {
		this.songTitle.setText(songTitle);
		this.albumTitle.setText(albumTitle);
		if (albumId != null) {
			imageDownloader.getArt(albumId, albumArt);
		}
		bindTimeViews(0, 0);
		this.progress.setMax(100);
		this.progress.setProgress(0);
	}
	
	public void bindTimeViews(Integer currentTime, Integer totalTime) {
		if (currentTime != null) this.currentTime.setText(UIUtils.formatDuration(currentTime));
		if (totalTime != null) this.totalTime.setText(UIUtils.formatDuration(totalTime));
	}
	
	public void setCurrentTime(int time) {
		this.currentTime.setText(UIUtils.formatDuration(time));
	}
}
