package com.ubuntuone.android.music.provider.dao;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.net.Uri;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.provider.MusicProvider;
import com.ubuntuone.api.music.model.U1Album;

public class AlbumDao extends Dao
{
	public AlbumDao() {
	}
	
	/**
	 * Updates the album. If the album doesn't exist, inserts the album.
	 * Backed by {@link MusicProvider}.
	 * 
	 * @param resolver
	 *            The content resolver.
	 * @param artist
	 *            The {@link U1Album} to save.
	 */
	public void updateOrInsert(ContentResolver resolver, U1Album album) {
		ContentValues values = MusicContentValues.forAlbum(album);
		Uri albumUri = Albums.buildAlbumUri(album.getId());
		int count = resolver.update(albumUri, values, null, null);
		if (count == 0) {
			resolver.insert(Albums.CONTENT_URI, values);
		}
	}
	
	/**
	 * Updates the album. If the album doesn't exist, inserts the album.
	 * Backed by {@link MusicDatabase}.
	 * 
	 * @param db
	 *            The {@link MusicDatabase} instance.
	 * @param artist
	 *            The {@link U1Album} to save.
	 * @return True if update or insert succeeded, false otherwise.
	 */
	public boolean updateOrInsert(SQLiteDatabase db, U1Album album) {
		boolean isSuccess = false;
		
		SQLiteStatement sqlStmt = db.compileStatement(getUpdateSql());
		int i = 1; // Index to the parameter to bind.
		sqlStmt.bindLong(i++, System.currentTimeMillis());
		
		sqlStmt.bindString(i++, album.getTitle());
		sqlStmt.bindString(i++, album.getArtist());
		sqlStmt.bindString(i++, album.getArtistId());
		sqlStmt.bindLong(i++, album.getYear());
		
		sqlStmt.bindLong(i++, album.getParsedDate());
		sqlStmt.bindString(i++, album.getAlbumUrl());
		sqlStmt.bindString(i++, album.getAlbumArtUrl());
		
		sqlStmt.bindString(i++, album.getId());
		// Can't use executeUpdateDelete on API < 11
		sqlStmt.execute();
		long rowsUpdated = getChanges(db);
		
		if (rowsUpdated == 0L) {
			sqlStmt = db.compileStatement(getInsertSql());
			i = 1;
			sqlStmt.bindLong(i++, System.currentTimeMillis());
			sqlStmt.bindString(i++, album.getId());
			
			sqlStmt.bindString(i++, album.getTitle());
			sqlStmt.bindString(i++, album.getArtist());
			sqlStmt.bindString(i++, album.getArtistId());
			sqlStmt.bindLong(i++, album.getYear());
			
			sqlStmt.bindLong(i++, album.getParsedDate());
			sqlStmt.bindString(i++, album.getAlbumUrl());
			sqlStmt.bindString(i++, album.getAlbumArtUrl());
			
			long rowId = sqlStmt.executeInsert();
			isSuccess = rowId != -1;
		} else {
			isSuccess = rowsUpdated == 1;
		}
		return isSuccess;
	}

	@Override
	public String onGetInsertSql() {
		return "INSERT INTO " + Tables.ALBUMS + "(" +
				Albums.UPDATED + ", " +
				Albums.ALBUM_ID + ", " +
				
				Albums.ALBUM_TITLE + ", " +
				Albums.ALBUM_ARTIST + ", " +
				Albums.ALBUM_ARTIST_ID + ", " +
				Albums.ALBUM_YEAR + ", " +
				
				Albums.ALBUM_DATE_ADDED + ", " +
				Albums.ALBUM_URL + ", " +
				Albums.ALBUM_ART_URL +
				
				") VALUES (?, ?, " +
				"?, ?, ?, ?, " +
				"?, ?, ?" +
				")";
	}

	@Override
	public String onGetUpdateSql() {
		return "UPDATE " + Tables.ALBUMS + " SET " +
				Albums.UPDATED + "=?, " +
				
				Albums.ALBUM_TITLE + "=?, " +
				Albums.ALBUM_ARTIST + "=?, " +
				Albums.ALBUM_ARTIST_ID + "=?, " +
				Albums.ALBUM_YEAR + "=?, " +
				
				Albums.ALBUM_DATE_ADDED + "=?, " +
				Albums.ALBUM_URL + "=?, " +
				Albums.ALBUM_ART_URL + "=? " +
				
				"WHERE " + Albums.ALBUM_ID + "=?";
	}
}
