package com.ubuntuone.android.music.provider.dao;

import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;

public abstract class Dao
{
	private String mInsertSql;
	private String mUdpateSql;
	
	/**
	 * Performs a lazy initialization with {@code Dao#onGetInsertSql()}.
	 * 
	 * @return The cached SQL INSERT string.
	 */
	public String getInsertSql() {
		if (mInsertSql == null) {
			mInsertSql = onGetInsertSql();
		}
		return mInsertSql;
	}
	
	/**
	 * Performs a lazy initialization with {@code Dao#onGetUpdateSql()}.
	 * 
	 * @return The cached SQL UPDATE string.
	 */
	public String getUpdateSql() {
		if (mUdpateSql == null) {
			mUdpateSql = onGetUpdateSql();
		}
		return mUdpateSql;
	}
	
	public long getChanges(SQLiteDatabase db) {
		SQLiteStatement countStmt = db.compileStatement("SELECT changes()");
		return countStmt.simpleQueryForLong();
	}
	
	/**
	 * Override to return the SQL INSERT string.
	 * 
	 * @return The SQL INSERT string.
	 */
	public abstract String onGetInsertSql();
	
	/**
	 * Override to return the SQL UPDATE string.
	 * 
	 * @return The SQL UPDATE string.
	 */
	public abstract String onGetUpdateSql();
}
