package com.ubuntuone.android.music.provider.dao;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.provider.MusicProvider;
import com.ubuntuone.api.music.model.U1Song;

public class GenreDao extends Dao
{
	public GenreDao() {
	}
	
	/**
	 * Inserts song genre. Backed by {@link MusicProvider}.
	 * 
	 * @param resolver
	 *            The content resolver.
	 * @param song
	 *            The {@link U1Song} of which genre to save.
	 */
	public void insert(ContentResolver resolver, U1Song song) {
		String genre = song.getGenre();
		String genreId = MusicContentValues.getGenreId(genre);
		if (genreId != null) {
			ContentValues genreValues = MusicContentValues.forGenre(genre.trim());
			resolver.insert(Genres.CONTENT_URI, genreValues);
		}
	}
	
	/**
	 * Inserts song genre. Backed by {@link MusicDatabase}.
	 * 
	 * @param db
	 *            The {@link MusicDatabase} instance.
	 * @param song
	 *            The {@link U1Song} of which genre to save.
	 * @return Always true for genre.
	 */
	public boolean insert(SQLiteDatabase db, String genre) {
		final SQLiteStatement genreInsert = db.compileStatement(getInsertSql());
		String genreId = MusicContentValues.getGenreId(genre);
		if (genreId == null) {
			// Ignore crappy genre. We're good.
			return true;
		}
		
		int j = 1; // Index to the parameter to bind.
		genreInsert.bindLong(j++, System.currentTimeMillis());
		genreInsert.bindString(j++, genreId);
		genreInsert.bindString(j++, genre);
		
		genreInsert.executeInsert();
		// Assume genre was inserted, we don't care.
		return true;
	}

	@Override
	public String onGetInsertSql() {
		return "INSERT INTO " + Tables.GENRES + "(" +
				Genres.UPDATED + ", " +
				Genres.GENRE_ID + ", " +
				Genres.GENRE_TITLE + 
				") VALUES (?, ?, ?)";
	}

	@Override
	public String onGetUpdateSql() {
		// Not used for genre.
		return null;
	}
}
