package com.ubuntuone.android.music.provider.dao;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.net.Uri;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.provider.MusicProvider;
import com.ubuntuone.api.music.model.U1Playlist;

public class PlaylistDao extends Dao
{
	public PlaylistDao() {
	}
	
	/**
	 * Updates the playlist. If the playlist doesn't exist, inserts the playlist.
	 * Backed by {@link MusicProvider}.
	 * 
	 * @param resolver
	 *            The content resolver.
	 * @param playlist
	 *            The {@link U1Playlist} to save.
	 */
	public void updateOrInsert(ContentResolver resolver, U1Playlist playlist) {
		ContentValues values = MusicContentValues.forPlaylist(playlist);
		Uri playlistUri = Playlists.buildPlaylistUri(playlist.getId());
		int count = resolver.update(playlistUri, values, null, null);
		if (count == 0) {
			resolver.insert(Playlists.CONTENT_URI, values);
		}
	}
	
	/**
	 * Updates the playlist. If the playlist doesn't exist, inserts the playlist.
	 * Backed by {@link MusicDatabase}.
	 * 
	 * @param db
	 *            The {@link MusicDatabase} instance.
	 * @param playlist
	 *            The {@link U1Playlist} to save.
	 * @return True if update or insert succeeded, false otherwise.
	 */
	public boolean updateOrInsert(SQLiteDatabase db, U1Playlist playlist) {
		boolean isSuccess = false;
		
		SQLiteStatement sqlStmt = db.compileStatement(getUpdateSql());
		int i = 1; // Index to the parameter to bind.
		
		sqlStmt.bindLong(i++, System.currentTimeMillis());
		
		sqlStmt.bindString(i++, playlist.getName());
		sqlStmt.bindLong(i++, playlist.getSongCount());
		sqlStmt.bindString(i++, playlist.getPlaylistUrl());
		
		sqlStmt.bindString(i++, playlist.getId());
		// Can't use executeUpdateDelete on API < 11
		sqlStmt.execute();
		long rowsUpdated = getChanges(db);
		
		if (rowsUpdated == 0L) {
			sqlStmt = db.compileStatement(getInsertSql());
			i = 1;
			sqlStmt.bindLong(i++, System.currentTimeMillis());
		
			sqlStmt.bindString(i++, playlist.getId());
			sqlStmt.bindString(i++, playlist.getName());
			sqlStmt.bindLong(i++, playlist.getSongCount());
			sqlStmt.bindString(i++, playlist.getPlaylistUrl());
			
			long rowId = sqlStmt.executeInsert();
			isSuccess = rowId != -1;
		} else {
			isSuccess = rowsUpdated == 1;
		}
		return isSuccess;
	}

	@Override
	public String onGetInsertSql() {
		return "INSERT INTO " + Tables.PLAYLISTS + "(" +
				Playlists.UPDATED + ", " +
				Playlists.PLAYLIST_ID + ", " +
				
				Playlists.PLAYLIST_NAME + ", " +
				Playlists.PLAYLIST_SONG_COUNT + ", " +
				Playlists.PLAYLIST_URL +
				
				") VALUES (?, ?, " +
				"?, ?, ?" +
				")";
	}

	@Override
	public String onGetUpdateSql() {
		return "UPDATE " + Tables.PLAYLISTS + " SET " +
				Playlists.UPDATED + "=?, " +
				Playlists.PLAYLIST_NAME + "=?, " +
				Playlists.PLAYLIST_SONG_COUNT + "=?, " +
				Playlists.PLAYLIST_URL + "=? " +
				"WHERE " + Playlists.PLAYLIST_ID + "=?";
		
	}
}
