package com.ubuntuone.android.music.provider.dao;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.net.Uri;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.provider.MusicProvider;
import com.ubuntuone.api.music.model.U1Song;

public class PlaylistSongDao extends Dao
{
	public PlaylistSongDao() {
	}
	
	/**
	 * Updates the playlist song. If the playlist song doesn't exist, inserts
	 * the playlist song. Backed by {@link MusicProvider}.
	 * 
	 * @param resolver
	 *            The content resolver.
	 * @param playlistId
	 *            The playlist id of the playlist to save the song to.
	 * @param song
	 *            The {@link U1Song} to save.
	 */
	public void updateOrInsert(ContentResolver resolver, String playlistId, U1Song song) {
		ContentValues values = MusicContentValues.forPlaylistSong(playlistId, song.getId());
		Uri playlistSongsUri = Playlists.buildPlaylistSongUri(playlistId, song.getId());
		int count = resolver.update(playlistSongsUri, values, null, null);
		if (count == 0) {
			resolver.insert(playlistSongsUri, values);
		}
	}
	
	/**
	 * Updates the playlist song. If the playlist song doesn't exist, inserts
	 * the playlist song. Backed by {@link MusicDatabase}.
	 * 
	 * @param db
	 *            The {@link MusicDatabase} instance.
	 * @param playlistId
	 *            The playlist id of the playlist to save the song to.
	 * @param song
	 *            The {@link U1Song} to save.
	 * @return True if update or insert succeeded, false otherwise.
	 */
	public boolean updateOrInsert(SQLiteDatabase db, String playlistId, String songId) {
		boolean isSuccess = false;
		
		SQLiteStatement sqlStmt = db.compileStatement(getUpdateSql());
		int i = 1; // Index to the parameter to bind.
		sqlStmt.bindLong(i++, System.currentTimeMillis());
		sqlStmt.bindString(i++, playlistId);
		sqlStmt.bindString(i++, songId);
		// Can't use executeUpdateDelete on API < 11
		sqlStmt.execute();
		long rowsUpdated = getChanges(db);
		
		if (rowsUpdated == 0L) {
			sqlStmt = db.compileStatement(getInsertSql());
			i = 1;
			sqlStmt.bindLong(i++, System.currentTimeMillis());
			sqlStmt.bindString(i++, playlistId);
			sqlStmt.bindString(i++, songId);
			
			long rowId = sqlStmt.executeInsert();
			isSuccess = rowId != -1;
		} else {
			isSuccess = rowsUpdated == 1;
		}
		return isSuccess;
	}

	@Override
	public String onGetInsertSql() {
		return "INSERT INTO " + Tables.PLAYLISTS_SONGS + "(" +
				PlaylistSongs.UPDATED + ", " +
				PlaylistSongs.PLAYLIST_ID + ", " +
				PlaylistSongs.SONG_ID +
				") VALUES (?, ?, ?)";
	}

	@Override
	public String onGetUpdateSql() {
		return "UPDATE " + Tables.PLAYLISTS_SONGS + " SET " +
				PlaylistSongs.UPDATED + "=? " +
				"WHERE " + PlaylistSongs.PLAYLIST_ID + "=? AND " + Songs.SONG_ID + "=?";
	}
}
