/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.util.U1ImageDownloader;
import com.ubuntuone.android.music.R;

public class AlbumActivity extends SherlockFragmentActivity implements OnClickListener
{
	private static final String TAG = makeLogTag(AlbumActivity.class);
	
	public static final String EXTRA_ALBUM_ID = Albums.ALBUM_ID;
	public static final String EXTRA_ALBUM_TITLE = Albums.ALBUM_TITLE;
	public static final String EXTRA_ALBUM_ARTIST = Albums.ALBUM_ARTIST;
	public static final String EXTRA_ALBUM_YEAR = Albums.ALBUM_YEAR;
	
	private U1ImageDownloader mImageDownloader = U1ImageDownloader.getAlbumArtDownloader();
	
	private String mAlbumId;
	private ImageView mAlbumArt;
	private TextView mAlbumTitle;
	private TextView mArtistAndYear;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_collection);
		
		mAlbumArt = (ImageView) findViewById(R.id.image);
		mAlbumTitle = (TextView) findViewById(R.id.text1);
		mArtistAndYear = (TextView) findViewById(R.id.text2);
		
		Button shuffleAllButton = (Button) findViewById(R.id.shuffle_all);
		shuffleAllButton.setOnClickListener(this);
		Button repeatAllButton = (Button) findViewById(R.id.repeat_all);
		repeatAllButton.setOnClickListener(this);
		
		mAlbumId = getIntent().getStringExtra(EXTRA_ALBUM_ID);
		if (mAlbumId == null) {
			throw new IllegalStateException("EXTRA_ALBUM_ID must not be null");
		}
		
		LoaderManager lm = getSupportLoaderManager();
		Loader<?> loader = lm.getLoader(R.id.album_songs_loader_id);
		if (loader != null) {
			loader.stopLoading();
		}
		mImageDownloader.getArt(mAlbumId, mAlbumArt);
		
		SongsFragment albumSongs = new SongsFragment();
		
		Intent intent = getIntent();
		String albumTitle = intent.getStringExtra(EXTRA_ALBUM_TITLE);
		mAlbumTitle.setText(albumTitle);
		String artistName = intent.getStringExtra(EXTRA_ALBUM_ARTIST);
		int albumYear = intent.getIntExtra(EXTRA_ALBUM_YEAR, 0);
		String albumYearStr =
				albumYear != 0 ? " - " + String.valueOf(albumYear) : "";
		mArtistAndYear.setText(String.format("%s%s", artistName, albumYearStr));
		
		Bundle args = new Bundle();
		args.putString(EXTRA_ALBUM_ID, mAlbumId);
		args.putString(EXTRA_ALBUM_TITLE, albumTitle);
		albumSongs.setArguments(args);
		
		ActionBar ab = getSupportActionBar();
		ab.setTitle(albumTitle);
		ab.setDisplayHomeAsUpEnabled(true);
		
		FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
		ft.replace(R.id.items, albumSongs);
		ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
		ft.commit();
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// Should start intent for home / artist albums?
			finish();
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}
	
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.shuffle_all:
			shuffleAll(v);
			break;
		case R.id.repeat_all:
			repeatAll(v);
			break;
		}
	}
	
	public void shuffleAll(View v) {
		Uri albumUri = Albums.buildAlbumUri(mAlbumId);
		MusicProviderUtils.clearAndEnqueueAlbum(this, albumUri);
		final boolean shuffle = Boolean.TRUE;
		final boolean noRepeat = Boolean.FALSE;
		PlayerActivity.startFrom(this, shuffle, noRepeat);
	}
	
	public void repeatAll(View v) {
		Uri albumUri = Albums.buildAlbumUri(mAlbumId);
		MusicProviderUtils.clearAndEnqueueAlbum(this, albumUri);
		final boolean noShuffle = Boolean.FALSE;
		final boolean repeat = Boolean.TRUE;
		PlayerActivity.startFrom(this, noShuffle, repeat);
	}
	
	public static void startFrom(Context context, Cursor cursor) {
		if (cursor == null) {
			Log.w(TAG, "null cursor in AlbumActivity.startFrom()");
			return;
		}
		
		String albumId = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ID));
		String title = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_TITLE));
		String artist = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ARTIST));
		int year = cursor.getInt(cursor.getColumnIndex(Albums.ALBUM_YEAR));
		
		startFrom(context, albumId, title, artist, year);
	}
	
	public static void startFrom(Context context, String albumId, String title,
			String artist, int year) {
		if (albumId == null) {
			throw new IllegalArgumentException("AlbumActivity: albumId must not be null");
		}
		
		Intent intent = new Intent(context, AlbumActivity.class);
		intent.putExtra(EXTRA_ALBUM_ID, albumId);
		intent.putExtra(EXTRA_ALBUM_TITLE, title);
		intent.putExtra(EXTRA_ALBUM_ARTIST, artist);
		intent.putExtra(EXTRA_ALBUM_YEAR, year);
		
		Log.d(TAG, "Launching AlbumActivity for albumId " + albumId);
		context.startActivity(intent);
	}
}
