/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.ui.ArtistActivity.EXTRA_ARTIST_ID;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ListView;

import com.actionbarsherlock.view.Menu;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.U1AlbumAdapter;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.util.UIUtils;

public class AlbumsFragment extends LoadingFragment<U1AlbumAdapter>
{
	private static final String TAG = makeLogTag(AlbumsFragment.class);
	
	private String mArtistId;

	@Override
	public int onGetHeaderResId() {
		return R.string.albums_title;
	}

	@Override
	public int onGetLoaderId() {
		return R.id.album_loader_id;
	}

	@Override
	public int onGetFragmentLayoutResId() {
		return R.layout.fragment_artists;
	}

	@Override
	public int onGetEmptyListResId() {
		return R.string.albums_no_found;
	}

	@Override
	public int onGetLoadingListResId() {
		return R.string.albums_loading;
	}
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		Bundle fragmentArgs = getArguments();
		if (fragmentArgs != null) {
			mArtistId = fragmentArgs.getString(EXTRA_ARTIST_ID);
		}
	}
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		final View contentView =  super.onCreateView(inflater, container, savedInstanceState);
		
		if (mArtistId != null) {
			View header = contentView.findViewById(R.id.header);
			header.setVisibility(View.GONE);
		}
		
		return contentView;
	}
	
	private void clearQueueAndPlay(ListView l, int position) {
		Cursor cursor = (Cursor) l.getAdapter().getItem(position);
		if (cursor != null && !cursor.isClosed()) {
			String albumId = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ID));
			if (albumId != null && isAdded()) {
				Uri albumUri = Albums.buildAlbumUri(albumId);
				Context context = getSherlockActivity();
				MusicProviderUtils.clearAndEnqueueAlbum(context, albumUri);
				PlayerActivity.startFrom(context, true);
			}
		}
	}

	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		showAlbumSongs(position);
	}

	private void showAlbumSongs(int position) {
		Cursor cursor = mAdapter.getCursor();
		cursor.moveToPosition(position);
		AlbumActivity.startFrom(getActivity(), cursor);
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		if (v.getId() == android.R.id.list) {
			menu.add(Menu.NONE, R.id.context_play, 0, R.string.description_play);
			
			menu.add(Menu.NONE, R.id.context_star, 0, R.string.description_star);
			Activity activity = getSherlockActivity();
			if (activity != null && UIUtils.isNetworkConnected(activity)) {
				menu.add(Menu.NONE, R.id.context_add_to_playlist, 0,
						R.string.description_add_to_playlist);
			}
			menu.add(Menu.NONE, R.id.context_add_to_queue, 0, R.string.description_add_to_queue);
		} else {
			super.onCreateContextMenu(menu, v, menuInfo);
		}
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {
		if (!getUserVisibleHint()) return false;
		
		AdapterView.AdapterContextMenuInfo info =
				(AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
		int position = info.position;
		
		switch (item.getItemId()) {
		case R.id.context_play:
			return onPlayContextItemSelected(position);
		case R.id.context_star:
			return onStarContextItemSelected(position);
		case R.id.context_add_to_playlist:
			return onAddToPlaylistContextItemSelected(position);
		case R.id.context_add_to_queue:
			return onAddToPlayQueueContextItemSelected(position);
		default:
			return super.onContextItemSelected(item);
		}
	}
	
	private boolean onPlayContextItemSelected(int position) {
		clearQueueAndPlay(mListView, position);
		return true;
	}
	
	private boolean onStarContextItemSelected(int position) {
		Cursor cursor = (Cursor) mListView.getItemAtPosition(position);
		if (cursor != null && !cursor.isClosed()) {
			String albumId = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ID));
			if (albumId != null && isAdded()) {
				Uri albumUri = Albums.buildAlbumUri(albumId);
				Context context = getSherlockActivity();
				
				MusicProviderUtils.toggleAlbumStarred(context, albumUri);
				
				int loaderId = getLoaderId();
				LoaderManager lm = getSherlockActivity().getSupportLoaderManager();
				lm.getLoader(loaderId).forceLoad();
			}
		}
		return true;
	}
	
	private boolean onAddToPlaylistContextItemSelected(int position) {
		Cursor cursor = (Cursor) mListView.getItemAtPosition(position);
		String albumId = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ID));
		Uri albumUri = Albums.buildAlbumUri(albumId);
		
		Intent intent = new Intent(getActivity(), PlaylistsActivity.class);
		intent.putExtra(PlaylistsActivity.EXTRA_ENQUEUE_CONTENT_ID, albumUri);
		startActivity(intent);
		return true;
	}
	
	private boolean onAddToPlayQueueContextItemSelected(int position) {
		Cursor cursor = (Cursor) mListView.getItemAtPosition(position);
		String albumId = cursor.getString(cursor.getColumnIndex(Albums.ALBUM_ID));
		Uri albumUri = Albums.buildAlbumUri(albumId);
		Activity activity = getSherlockActivity();
		if (activity != null) MusicProviderUtils.enqueueAlbum(activity, albumUri);
		return true;
	}

	@Override
	public U1AlbumAdapter onCreateAdapter(Context context, Cursor cursor) {
		Log.d(TAG, "AlbumsFragment#onCreateAdapter");
		
		return new U1AlbumAdapter(context, cursor);
	}

	@Override
	public Loader<Cursor> onCreateLoader(int id, Bundle args) {
		Log.d(TAG, "AlbumsFragment#onCreateLoader");
		
		CursorLoader loader;
		if (mArtistId == null) {
			loader = new CursorLoader(getSherlockActivity(), Albums.CONTENT_URI,
					Albums.DEFAULT_PROJECTION, null, null, Albums.DEFAULT_SORT);
		} else {
			Uri artistUri = Artists.buildAlbumsUri(mArtistId);
			loader = new CursorLoader(getSherlockActivity(), artistUri,
					Albums.DEFAULT_PROJECTION, null, null, Albums.DEFAULT_SORT);
		}
		setIsLoading(true);
		return loader;
	}
}
