/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.util.U1ImageDownloader;

public class ArtistActivity extends SherlockFragmentActivity implements OnClickListener
{
	private static final String TAG = makeLogTag(ArtistActivity.class);
	
	public static final String EXTRA_ARTIST_ID = Artists.ARTIST_ID;
	public static final String EXTRA_ARTIST_NAME = Artists.ARTIST_NAME;
	public static final String EXTRA_ALBUM_COUNT = Artists.ALBUM_COUNT;
	public static final String EXTRA_SONG_COUNT = Artists.SONG_COUNT;
	
	private U1ImageDownloader mImageDownloader = U1ImageDownloader.getArtistArtDownloader();
	
	private String mArtistId;
	private ImageView mArtistArt;
	private TextView mArtistName;
	private TextView mAlbumsCount;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_collection);
		
		mArtistArt = (ImageView) findViewById(R.id.image);
		mArtistName = (TextView) findViewById(R.id.text1);
		mAlbumsCount = (TextView) findViewById(R.id.text2);
		
		Button shuffleAllButton = (Button) findViewById(R.id.shuffle_all);
		shuffleAllButton.setOnClickListener(this);
		Button repeatAllButton = (Button) findViewById(R.id.repeat_all);
		repeatAllButton.setOnClickListener(this);

		mArtistId = getIntent().getStringExtra(EXTRA_ARTIST_ID);
		if (mArtistId == null) {
			throw new IllegalStateException("ArtistActvity#onCreate: " +
					"null EXTRA_ARTIST_ID extra provided");
		}

		LoaderManager lm = getSupportLoaderManager();
		Loader<?> loader = lm.getLoader(R.id.artist_albums_loader_id);
		if (loader != null) {
			loader.stopLoading();
		}
		mImageDownloader.getArt(mArtistId, mArtistArt);

		// TODO recycle fragment.
		AlbumsFragment artistAlbums = new AlbumsFragment();

		String artistName = getIntent().getStringExtra(EXTRA_ARTIST_NAME);
		mArtistName.setText(artistName);
		int artistAlbumCount = getIntent().getIntExtra(EXTRA_ALBUM_COUNT, 0);
		int artistSongCount = getIntent().getIntExtra(EXTRA_SONG_COUNT, 0);
		mAlbumsCount.setText(getString(R.string.artist_stats,
				artistAlbumCount, artistSongCount));
		
		Bundle args = new Bundle();
		args.putString(EXTRA_ARTIST_ID, mArtistId);
		artistAlbums.setArguments(args);

		ActionBar ab = getSupportActionBar();
		ab.setTitle(artistName);
		ab.setDisplayHomeAsUpEnabled(true);

		FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
		ft.replace(R.id.items, artistAlbums);
		ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
		ft.commit();
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// Should rather start intent for home with clear top?
			finish();
			return true;

		default:
			return super.onOptionsItemSelected(item);
		}
	}
	
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.shuffle_all:
			shuffleAll(v);
			break;
		case R.id.repeat_all:
			repeatAll(v);
			break;
		}
	}
	
	public void shuffleAll(View v) {
		Uri artistUri = Artists.buildArtistUri(mArtistId);
		MusicProviderUtils.clearAndEnqueueArtist(this, artistUri);
		PlayerActivity.startFrom(this, Boolean.TRUE, Boolean.FALSE);
	}
	
	public void repeatAll(View v) {
		Uri artistUri = Artists.buildArtistUri(mArtistId);
		MusicProviderUtils.clearAndEnqueueArtist(this, artistUri);
		PlayerActivity.startFrom(this, Boolean.FALSE, Boolean.TRUE);
	}
	
	public static void startFrom(Context context, Cursor cursor) {
		if (cursor == null) {
			Log.w(TAG, "ArtistActivity#startFrom: null cursor provided");
			return;
		}
		
		String artistId = cursor.getString(cursor.getColumnIndex(Artists.ARTIST_ID));
		String artistName = cursor.getString(cursor.getColumnIndex(Artists.ARTIST_NAME));
		int artistAlbumCount = cursor.getInt(cursor.getColumnIndex(Artists.ALBUM_COUNT));
		int artistSongCount = cursor.getInt(cursor.getColumnIndex(Artists.SONG_COUNT));
		
		startFrom(context, artistId, artistName, artistAlbumCount, artistSongCount);
	}
	
	private static void startFrom(Context context, String artistId, String artistName,
			int artistAlbumCount, int artistSongCount) {
		if (artistId == null) {
			throw new IllegalArgumentException("ArtistActivity#startFrom: null artistId provided");
		}
		
		Intent intent = new Intent(context, ArtistActivity.class);
		intent.putExtra(EXTRA_ARTIST_ID, artistId);
		intent.putExtra(EXTRA_ARTIST_NAME, artistName);
		intent.putExtra(EXTRA_ALBUM_COUNT, artistAlbumCount);
		intent.putExtra(EXTRA_SONG_COUNT, artistSongCount);
		
		Log.d(TAG, "Launching ArtistActivity for artistId " + artistId);
		context.startActivity(intent);
	}
}
