/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ListView;

import com.actionbarsherlock.view.Menu;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.U1ArtistAdapter;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicProviderUtils;

public class ArtistsFragment extends LoadingFragment<U1ArtistAdapter>
{
	private static final String TAG = makeLogTag(ArtistsFragment.class);
	
	@Override
	public int onGetHeaderResId() {
		return R.string.artists_title;
	}

	@Override
	public int onGetLoaderId() {
		return R.id.artist_loader_id;
	}

	@Override
	public int onGetFragmentLayoutResId() {
		return R.layout.fragment_loading_list;
	}

	@Override
	public int onGetEmptyListResId() {
		return R.string.artists_no_found;
	}

	@Override
	public int onGetLoadingListResId() {
		return R.string.artists_loading;
	}
	
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		showArtistAlbums(position);
	}
	
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		registerForContextMenu(mListView);
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		if (v.getId() == android.R.id.list) {
			menu.add(Menu.NONE, R.id.context_play, 0, R.string.description_play);
			menu.add(Menu.NONE, R.id.context_add_to_queue, 0, R.string.description_add_to_queue);
		} else {
			super.onCreateContextMenu(menu, v, menuInfo);
		}
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
		if (!getUserVisibleHint()) return false;
		
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		Cursor cursor = (Cursor) mAdapter.getCursor();
		cursor.moveToPosition(info.position);
		
		String artistId = cursor.getString(cursor.getColumnIndex(Artists.ARTIST_ID));
		Uri artistUri = Artists.buildArtistUri(artistId);
		
		switch (item.getItemId()) {
		case R.id.context_play:
			MusicProviderUtils.clearAndEnqueueArtist(getActivity(), artistUri);
			PlayerActivity.startFrom(getActivity(), true);
			return true;
		case R.id.context_add_to_queue:
			MusicProviderUtils.enqueueArtist(getActivity(), artistUri);
			return true;
		default:
			return super.onContextItemSelected(item);
		}
	}
	
	private void showArtistAlbums(int position) {
		Cursor cursor = mAdapter.getCursor();
		cursor.moveToPosition(position);
		ArtistActivity.startFrom(getActivity(), cursor);
	}
	
	@Override
	public U1ArtistAdapter onCreateAdapter(Context context, Cursor cursor) {
		Log.d(TAG, "ArtistsFragment#onCreateAdapter");
		
		return new U1ArtistAdapter(context, cursor);
	}

	@Override
	public Loader<Cursor> onCreateLoader(int id, Bundle args) {
		Log.d(TAG, "ArtistsFragment#onCreateLoader");
		
		CursorLoader loader = new CursorLoader(getSherlockActivity(), Artists.CONTENT_URI,
				Artists.DEFAULT_PROJECTION, null, null, Artists.DEFAULT_SORT);
		setIsLoading(true);
		return loader;
	}
	
	@Override
	public boolean onOptionsItemSelected(com.actionbarsherlock.view.MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_shuffle_all:
			return onShuffleAllOptionItemSelected();
		default:
			return super.onOptionsItemSelected(item);
		}
	}
	
	private boolean onShuffleAllOptionItemSelected() {
		if (isAdded()) {
			HomeActivity activity = (HomeActivity) getSherlockActivity();
			if (activity != null) {
				activity.shuffleAllSongs();
			}
		}
		return true;
	}
}
