/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicProviderUtils;

public class GenreActivity extends SherlockFragmentActivity implements OnClickListener
{
	@SuppressWarnings("unused")
	private static final String TAG = makeLogTag(GenreActivity.class);
	
	public static final String EXTRA_GENRE_ID = Genres.GENRE_ID;
	
	private String mGenreId;
	private ImageView mGenreArt;
	private TextView mGenreTitle;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_collection);
		
		mGenreArt = (ImageView) findViewById(R.id.image);
		mGenreTitle = (TextView) findViewById(R.id.text1);
		
		Button shuffleAllButton = (Button) findViewById(R.id.shuffle_all);
		shuffleAllButton.setOnClickListener(this);
		Button repeatAllButton = (Button) findViewById(R.id.repeat_all);
		repeatAllButton.setOnClickListener(this);

		mGenreId = getIntent().getStringExtra(Genres.GENRE_ID);
		if (mGenreId == null) {
			throw new IllegalStateException("genre_id cannot be null");
		}

		LoaderManager lm = getSupportLoaderManager();
		Loader<?> loader = lm.getLoader(R.id.playlist_songs_loader_id);
		if (loader != null) {
			loader.stopLoading();
		}
		mGenreArt.setImageResource(R.drawable.default_art);

		Intent intent = getIntent();
		String genreTitle = intent.getStringExtra(Genres.GENRE_TITLE);
		mGenreTitle.setText(genreTitle);
		
		Bundle args = new Bundle();
		args.putString(Genres.GENRE_ID, mGenreId);
		args.putString(Genres.GENRE_TITLE, genreTitle);
		// TODO Recycle fragment?
		SongsFragment genreSongs = new SongsFragment();
		genreSongs.setArguments(args);

		ActionBar ab = getSupportActionBar();
		ab.setTitle(genreTitle);
		ab.setDisplayHomeAsUpEnabled(true);

		FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
		ft.replace(R.id.items, genreSongs);
		ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
		ft.commit();
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// Should start intent for home / artist albums?
			finish();
			return true;

		default:
			return super.onOptionsItemSelected(item);
		}
	}
	
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.shuffle_all:
			shuffleAll(v);
			break;
		case R.id.repeat_all:
			repeatAll(v);
			break;
		}
	}
	
	public void shuffleAll(View v) {
		Uri genreUri = Genres.buildGenreUri(mGenreId);
		MusicProviderUtils.clearAndEnqueueGenre(this, genreUri);
		final boolean shuffle = Boolean.TRUE;
		final boolean noRepeat = Boolean.FALSE;
		PlayerActivity.startFrom(this, shuffle, noRepeat);
	}
	
	public void repeatAll(View v) {
		Uri genreUri = Genres.buildGenreUri(mGenreId);
		MusicProviderUtils.clearAndEnqueueGenre(this, genreUri);
		final boolean noShuffle = Boolean.FALSE;
		final boolean repeat = Boolean.TRUE;
		PlayerActivity.startFrom(this, noShuffle, repeat);
	}
}
