/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.LOGD;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.View;
import android.widget.ListView;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.adapter.U1GenreAdapter;
import com.ubuntuone.android.music.provider.MusicContract.Genres;

public class GenresFragment extends LoadingFragment<U1GenreAdapter>
{
	private static final String TAG = makeLogTag(GenresFragment.class);
	
	@Override
	public int onGetHeaderResId() {
		return R.string.genres_title;
	}

	@Override
	public int onGetLoaderId() {
		return R.id.genre_loader_id;
	}

	@Override
	public int onGetFragmentLayoutResId() {
		return R.layout.fragment_loading_list;
	}

	@Override
	public int onGetEmptyListResId() {
		return R.string.genres_no_found;
	}

	@Override
	public int onGetLoadingListResId() {
		return R.string.genres_loading;
	}
	
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		showGenreSongs(position);
	}
	
	private void showGenreSongs(int position) {
		Cursor cursor = mAdapter.getCursor();
		cursor.moveToPosition(position);
		
		String genreId = cursor.getString(cursor.getColumnIndex(Genres.GENRE_ID));
		String genreTitle = cursor.getString(cursor.getColumnIndex(Genres.GENRE_TITLE));
		
		LOGD(TAG, "Launching GenreActivity for genre_id = " + genreId);
		
		Intent intent = new Intent(getActivity(), GenreActivity.class);
		intent.putExtra(Genres.GENRE_ID, genreId);
		intent.putExtra(Genres.GENRE_TITLE, genreTitle);
		startActivity(intent);
    }

	@Override
	public U1GenreAdapter onCreateAdapter(Context context, Cursor cursor) {
		Log.d(TAG, "GenresFragment#onCreateAdapter");
		
		return new U1GenreAdapter(context, cursor);
	}

	@Override
	public Loader<Cursor> onCreateLoader(int id, Bundle args) {
		Log.d(TAG, "GenresFragment#onCreateLoader");
		
		CursorLoader loader = new CursorLoader(getSherlockActivity(), Genres.CONTENT_URI,
				Genres.DEFAULT_PROJECTION, null, null, Genres.DEFAULT_SORT);
		setIsLoading(true);
		return loader;
	}
}
