/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012-2013 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.annotation.TargetApi;
import android.content.ComponentName;
import android.content.ContentResolver;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.res.Configuration;
import android.database.Cursor;
import android.media.AudioManager;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.ResultReceiver;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.support.v4.view.ViewPager;
import android.support.v4.widget.CursorAdapter;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Adapter;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.android.music.adapter.U1PlaylistAdapter;
import com.ubuntuone.android.music.adapter.U1PlaylistAdapter.Position;
import com.ubuntuone.android.music.adapter.holder.NowPlayingViewHolder;
import com.ubuntuone.android.music.model.PlayerState;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.service.MusicService;
import com.ubuntuone.android.music.service.MusicServiceBinder;
import com.ubuntuone.android.music.service.MusicServiceCallback;
import com.ubuntuone.android.music.service.SyncService;
import com.ubuntuone.android.music.ui.PlaylistsFragment.OnPlaylistSelectedListener;
import com.ubuntuone.android.music.ui.dialog.InputAlertDialog.InputAlertDialogListener;
import com.ubuntuone.android.music.util.AccountUtils;
import com.ubuntuone.android.music.util.NotAuthenticatedException;
import com.ubuntuone.android.music.util.U1ImageDownloader;
import com.ubuntuone.android.music.util.UIUtils;

public class HomeActivity extends SherlockFragmentActivity implements ActionBar.TabListener,
		ViewPager.OnPageChangeListener, OnClickListener, OnPlaylistSelectedListener,
		InputAlertDialogListener, MusicServiceCallback
{
	@SuppressWarnings("unused")
	private static final String TAG = makeLogTag(HomeActivity.class);

	private static final String EXTRA_NAV_POSITION = "extra_tab";

	private static final int REQUEST_SIGN_IN = 0;
	
	/* package */ static final int PLEASE_WAIT_DIALOG_ID = 1;
	/* package */ static final int ERROR_DIALOG_ID = 2;
	/* package */ static final int CREATE_PLAYLIST_DIALOG_ID = 3;
	
	private Handler mHandler = new Handler();
	private ResultReceiver mSyncReceiver = new SyncResultReceiver(mHandler);
	
	private ServiceConnection mConn;
	private boolean mIsBound = false;
	private MusicService mService;
	
	private U1ImageDownloader mImageDownloader;

	private ActionBar mActionBar;
	private ViewPager mViewPager;
	private Menu mOptionsMenu;
	
	private NowPlayingViewHolder mNowPlaying;
	
	private ArtistsFragment mArtistFragment;
	private AlbumsFragment mAlbumFragment;
	private SongsFragment mSongFragment;
	private PlaylistsFragment mPlaylistFragment;
	private GenresFragment mGenreFragment;
	
	private Toast mShuffleAllToast;
	
	/* package */ boolean _shuffleAllSongsTaskLaunched = false;
	private ShuffleAllSongsTask mShuffleAllSongsTask;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setVolumeControlStream(AudioManager.STREAM_MUSIC);
		
		UIUtils.enableDisableActivities(this);
		setContentView(R.layout.activity_home);
		FragmentManager fm = getSupportFragmentManager();

		mViewPager = (ViewPager) findViewById(R.id.pager);
		if (mViewPager != null) {
			mViewPager.setAdapter(new HomePagerAdapter(fm));
			mViewPager.setOnPageChangeListener(this);
		}

		setUpActionBar();
		setUpTabs();

		restoreState(savedInstanceState);

		if (!AccountUtils.isAuthenticated(this)) {
			AccountUtils.startAuthenticationFlow(this, REQUEST_SIGN_IN);
			return;
		}
		
		try {
			AccountUtils.ensureAccountExists(this, AccountUtils.getChosenAccountName(this));
		} catch (NotAuthenticatedException e) {
			AccountUtils.setChosenAccountName(this, null);
			AccountUtils.setAuthToken(this, null);
			AccountUtils.startAuthenticationFlow(this, REQUEST_SIGN_IN);
			return;
		}
			
		// Don't refresh on orientation changes.
		if (savedInstanceState == null && UIUtils.isNetworkConnected(this)) {
			SyncService.syncMusicMetadata(this, mSyncReceiver);
		}
		MusicService.startFrom(this);
		
		mImageDownloader = U1ImageDownloader.getAlbumArtDownloader();
	}
	
	private void setUpActionBar() {
		mActionBar = getSupportActionBar();
		mActionBar.setDisplayShowHomeEnabled(false);
		mActionBar.setDisplayShowTitleEnabled(false);
		mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);
		
		if (UIUtils.isTablet(this) || getResources().getConfiguration().orientation
				== Configuration.ORIENTATION_LANDSCAPE) {
			mActionBar.setDisplayOptions(
					ActionBar.DISPLAY_SHOW_CUSTOM, ActionBar.DISPLAY_SHOW_CUSTOM);
			View inflatedView = getLayoutInflater().inflate(R.layout.action_bar_view, null);
			setupNowPlayingViews(inflatedView);
			mActionBar.setCustomView(inflatedView);
		}
	}

	private void setUpTabs() {
		mActionBar.addTab(mActionBar.newTab()
				.setIcon(R.drawable.ic_action_help).setTabListener(this));
		mActionBar.addTab(mActionBar.newTab()
				.setIcon(R.drawable.ic_action_help).setTabListener(this));
		mActionBar.addTab(mActionBar.newTab()
				.setIcon(R.drawable.ic_action_help).setTabListener(this));
		mActionBar.addTab(mActionBar.newTab()
				.setIcon(R.drawable.ic_action_help).setTabListener(this));
		mActionBar.addTab(mActionBar.newTab()
				.setIcon(R.drawable.ic_action_help).setTabListener(this));
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
		case REQUEST_SIGN_IN:
			if (resultCode == RESULT_OK) {
				LoaderManager lm = getSupportLoaderManager();
				Loader<?> loader = lm.getLoader(R.id.artist_loader_id);
				if (loader != null) {
					loader.reset();
					loader.forceLoad();
				}
				SyncService.syncMusicMetadata(this, mSyncReceiver);
				MusicService.startFrom(this);
				
				mImageDownloader = U1ImageDownloader.getAlbumArtDownloader();
			} else {
				finish();
			}
			break;
			
		default:
			super.onActivityResult(requestCode, resultCode, data);
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		
		Intent service = new Intent(this, MusicService.class);
		mConn = new ServiceConnection() {
			@Override
			public void onServiceDisconnected(ComponentName name) {
				mIsBound = false;
				mService = null;
			}
			
			@SuppressWarnings("unchecked")
			@Override
			public void onServiceConnected(ComponentName name, IBinder service) {
				mService = ((MusicServiceBinder<MusicService>) service).getService();
				mIsBound = true;
				mService.register(HomeActivity.this);
			}
		};
		if (UbuntuOneMusic.SHOULD_BIND && !isFinishing()) {
			bindService(service, mConn, BIND_AUTO_CREATE | BIND_IMPORTANT);
		}
		
		// Refresh playlists song count, if necessary.
		int position = mViewPager.getCurrentItem();
		if (position == 3) {
			forcePlaylistsRefresh();
		}
	}

	@Override
	protected void onPause() {
		if (mConn != null && mIsBound) {
			unbindService(mConn);
		}
		
		if (mShuffleAllToast != null) mShuffleAllToast.cancel();
		super.onPause();
	}
	
	@Override
	protected void onStop() {
		if (mShuffleAllSongsTask != null && !mShuffleAllSongsTask.isCancelled()) {
			mShuffleAllSongsTask.cancel(true);
		}
		super.onStop();
	}

	@Override
	public void onTabSelected(ActionBar.Tab tab,
			FragmentTransaction fragmentTransaction) {
		mViewPager.setCurrentItem(tab.getPosition());
	}

	@Override
	public void onTabUnselected(ActionBar.Tab tab,
			FragmentTransaction fragmentTransaction) {
		// Do nothing.
	}

	@Override
	public void onTabReselected(ActionBar.Tab tab,
			FragmentTransaction fragmentTransaction) {
		// Do nothing.
	}

	@Override
	public void onPageScrolled(int i, float v, int i1) {
		// Do nothing.
	}

	@Override
	public void onPageSelected(int position) {
		getSupportActionBar().setSelectedNavigationItem(position);
		// TODO Track page views.
		
		if (position == 3) {
			forcePlaylistsRefresh();
		}
	}
	
	/* package */ void forcePlaylistsRefresh() {
		LoaderManager lm = getSupportLoaderManager();
		Loader<?> loader = lm.getLoader(R.id.playlist_loader_id);
		if (loader != null) loader.forceLoad();
	}

	@Override
	public void onPageScrollStateChanged(int i) {
		// Do nothing.
	}

	@Override
	protected void onSaveInstanceState(Bundle outState) {
		if (mActionBar != null) {
			int position = mActionBar.getSelectedNavigationIndex();
			outState.putInt(EXTRA_NAV_POSITION, position);
		}
		super.onSaveInstanceState(outState);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		mOptionsMenu = menu;
		getSupportMenuInflater().inflate(R.menu.home, menu);
		if (!UIUtils.isTablet(this) && getResources().getConfiguration().orientation
				== Configuration.ORIENTATION_PORTRAIT) {
			setupNowPlayingItem(menu);
		}
		setupSearchMenuItem(menu);
		
		if (SyncService.isSyncRunning()) {
			setRefreshActionButtonState(true);
		}
		return true;
	}
	
	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		// Hide 'Shuffle all' on playlists and genres pages.
		boolean isShuffleAllVisible = !(mViewPager.getCurrentItem() > HomePagerAdapter.SONGS_PAGE);
		MenuItem shuffleAll = menu.findItem(R.id.menu_shuffle_all);
		if (shuffleAll != null) {
			shuffleAll.setVisible(isShuffleAllVisible);
		}
		// Hide 'Sign out' on all pages but the first one.
		boolean isArtistsPage = mViewPager.getCurrentItem() == HomePagerAdapter.ARTISTS_PAGE;
		MenuItem signOut = menu.findItem(R.id.menu_sign_out);
		if (signOut != null) {
			signOut.setVisible(isArtistsPage);
		}
		return super.onPrepareOptionsMenu(menu);
	}
	
	private void setupNowPlayingItem(Menu menu) {
		LinearLayout actionView = (LinearLayout) menu.findItem(R.id.menu_now_playing)
				.getActionView();
        View inflatedView = getLayoutInflater().inflate(R.layout.action_bar_view, null);
        setupNowPlayingViews(inflatedView);
        actionView.addView(inflatedView);
	}

	private void setupNowPlayingViews(View nowPlayingView) {
		mNowPlaying = new NowPlayingViewHolder(nowPlayingView);
		mNowPlaying.nowPlaying.setOnClickListener(this);
	}

	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
	private void setupSearchMenuItem(Menu menu) {
		// MenuItem searchItem = menu.findItem(R.id.menu_search);
		// if (searchItem != null && UIUtils.hasHoneycomb()) {
		// SearchView searchView = (SearchView) searchItem.getActionView();
		// if (searchView != null) {
		// SearchManager searchManager = (SearchManager)
		// getSystemService(SEARCH_SERVICE);
		// searchView.setSearchableInfo(searchManager.getSearchableInfo(getComponentName()));
		// }
		// }
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_refresh:
			return onRefreshOptionItemSelected();
		case R.id.menu_search:
			return onSearchOptionItemSelected();
		case R.id.menu_shuffle_all:
			return onShuffleAllOptionItemSelected();
		case R.id.menu_settings:
			return onSettingsOptionItemSelected();
		case R.id.menu_help:
			return true;
		case R.id.menu_sign_out:
			return onSignOutOptionItemSelected();
		default:
			return super.onOptionsItemSelected(item);
		}
	}
	
	private boolean onRefreshOptionItemSelected() {
		if (UIUtils.isNetworkConnected(this)) {
			SyncService.syncMusicMetadata(this, mSyncReceiver);
		} else {
			Toast.makeText(this, R.string.no_network, Toast.LENGTH_LONG).show();
		}
		return true;
	}
	
	private boolean onSearchOptionItemSelected() {
		if (!UIUtils.hasHoneycomb()) {
			startSearch(null, false, Bundle.EMPTY, false);
			return true;
		}
		return false;
	}
	
	private boolean onShuffleAllOptionItemSelected() {
		shuffleAllSongs();
		return true;
	}
	
	private boolean onSettingsOptionItemSelected() {
		Intent settingsIntent = new Intent(this, PreferencesActivity.class);
		startActivity(settingsIntent);
		return true;
	}
	
	private boolean onSignOutOptionItemSelected() {
		AccountUtils.signOut(this);
		finish();
		return true;
	}

	private void restoreState(Bundle savedInstanceState) {
		if (savedInstanceState == null) return;

		int position = savedInstanceState.getInt(EXTRA_NAV_POSITION);
		mActionBar.setSelectedNavigationItem(position);
		
		if (SyncService.isSyncRunning()) {
			setRefreshActionButtonState(true);
		}
	}

	public void setRefreshActionButtonState(boolean isRefreshing) {
		if (mOptionsMenu == null) {
			return;
		}

		final MenuItem refreshItem = mOptionsMenu.findItem(R.id.menu_refresh);
		if (refreshItem != null) {
			refreshItem.setEnabled(!isRefreshing);
			if (isRefreshing) {
				refreshItem.setActionView(R.layout.actionbar_indeterminate_progress);
			} else {
				refreshItem.setActionView(null);
			}
		}
	}
	
	protected void onProgress(int loaderId) {
		Loader<Cursor> loader = getSupportLoaderManager().getLoader(loaderId);
		if (loader != null) {
			loader.onContentChanged();
		}
	}
	
	private class SyncResultReceiver extends ResultReceiver {
		public SyncResultReceiver(Handler handler) {
			super(handler);
		}

		@Override
		protected void onReceiveResult(int resultCode, Bundle resultData) {
			switch (resultCode) {
			case SyncService.SYNC_STARTED:
				setRefreshActionButtonState(true);
				break;
			case SyncService.SYNC_PROGRESS:
				int loaderId = resultData.getInt(SyncService.EXTRA_LOADER_ID);
				onProgress(loaderId);
				break;
			case SyncService.SYNC_FINISHED:
				// Force refresh in the grenres fragment.
				onProgress(R.id.genre_loader_id);
				setRefreshActionButtonState(false);
				break;
			default:
				super.onReceiveResult(resultCode, resultData);
			}
		}
	}

	private class HomePagerAdapter extends FragmentPagerAdapter
	{
		public static final int ARTISTS_PAGE = 0;
		public static final int ALBUMS_PAGE = 1;
		public static final int SONGS_PAGE = 2;
		public static final int PLAYLISTS_PAGE = 3;
		public static final int GENRES_PAGE = 4;
				
		public HomePagerAdapter(FragmentManager fm) {
			super(fm);
		}

		@Override
		public Fragment getItem(int position) {
			switch (position) {
			case ARTISTS_PAGE:
				if (mArtistFragment == null) {
					mArtistFragment = new ArtistsFragment();
				}
				return mArtistFragment;
			case ALBUMS_PAGE:
				if (mAlbumFragment == null) {
					mAlbumFragment = new AlbumsFragment();
				}
				return mAlbumFragment;
			case SONGS_PAGE:
				if (mSongFragment == null) {
					mSongFragment = new SongsFragment();
				}
				return mSongFragment;
			case PLAYLISTS_PAGE:
				if (mPlaylistFragment == null) {
					Bundle args = new Bundle();
					args.putBoolean(PlaylistsFragment.EXTRA_INCLUDE_CUSTOM, true);
					mPlaylistFragment = new PlaylistsFragment();
					mPlaylistFragment.setArguments(args);
				}
				return mPlaylistFragment;
			case GENRES_PAGE:
				if (mGenreFragment == null) {
					mGenreFragment = new GenresFragment();
				}
				return mGenreFragment;
			default:
				return null;
			}
		}

		@Override
		public int getCount() {
			return 5;
		}
	}
	
	public void shuffleAllSongs() {
		if (mShuffleAllSongsTask == null) {
			mShuffleAllSongsTask = new ShuffleAllSongsTask();
			mShuffleAllSongsTask.execute();
			_shuffleAllSongsTaskLaunched = true;
		} else {
			if (mShuffleAllToast != null) mShuffleAllToast.cancel();
			mShuffleAllToast = Toast.makeText(this,
					"Still working, just a sec...", Toast.LENGTH_LONG);
			mShuffleAllToast.show();
		}
	}
	
	private class ShuffleAllSongsTask extends AsyncTask<Void, Void, Void> {
		@Override
		protected void onPreExecute() {
			if (!isFinishing()) {
				if (mShuffleAllToast != null) mShuffleAllToast.cancel();
				mShuffleAllToast = Toast.makeText(HomeActivity.this,
						"Enqueuing all songs...", Toast.LENGTH_LONG);
				mShuffleAllToast.show();
			}
		}

		@Override
		protected Void doInBackground(Void... params) {
			if (!isCancelled()) {
				MusicProviderUtils.clearAndEnqueueAllSongs(HomeActivity.this);
			}
			return null;
		}
		
		@Override
		protected void onPostExecute(Void result) {
			if (!isFinishing()) {
				if (mShuffleAllToast != null) mShuffleAllToast.cancel();
				mShuffleAllToast = Toast.makeText(HomeActivity.this,
						"All songs queued.", Toast.LENGTH_SHORT);
				mShuffleAllToast.show();
			}
			mShuffleAllSongsTask = null;
		}
	}
	
	@Override
	public void onPlaylistSelected(Adapter adapter, int position, boolean hasHeaders) {
		if (hasHeaders && position < U1PlaylistAdapter.Position.COUNT) {
			switch (position) {
			case Position.STARRED: {
				int count = MusicProviderUtils.getStarredSongCount(this);
				PlaylistActivity.startFrom(this, MusicProviderUtils.STARRED_ID,
						getString(R.string.starred), count);
				break;
			}
			case Position.CACHED: {
				int count = MusicProviderUtils.getCachedSongCount(this);
				PlaylistActivity.startFrom(this, MusicProviderUtils.CACHED_ID,
						getString(R.string.cached), count);
				break;
			}
			case Position.QUEUED: {
				PlayerActivity.startWithQueue(this);
				break;
			}
			}
		} else {
			if (hasHeaders) position -= Position.COUNT;
			
			Cursor cursor = ((CursorAdapter) adapter).getCursor();
			cursor.moveToPosition(position);
			
			int songCount = cursor.getInt(cursor.getColumnIndex(Playlists.PLAYLIST_SONG_COUNT));
			if (songCount == 0) {
				String title = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_NAME));
				String text = getString(R.string.playlist_is_empty, title);
				if (!isFinishing()) {
					Toast.makeText(this, text, Toast.LENGTH_SHORT).show();
				}
			} else {
				PlaylistActivity.startFrom(this, cursor);
			}
		}
	}
	
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.now_playing:
			onNowPlayingClicked();
			break;
		}
	};
	
	@Override
	public void onPositiveButtonClicked(int dialogId, String text) {
		if (dialogId == CREATE_PLAYLIST_DIALOG_ID) {
			if (mPlaylistFragment != null) mPlaylistFragment.onPositiveButtonClicked(dialogId, text);
		}
	}

	@Override
	public void onNegativeButtonClicked(int dialogId) {
		if (dialogId == CREATE_PLAYLIST_DIALOG_ID) {
			if (mPlaylistFragment != null) mPlaylistFragment.onNegativeButtonClicked(dialogId);
		}
	}

	private void onNowPlayingClicked() {
		PlayerActivity.startFrom(this, null);
	}

	@Override
	public IBinder asBinder() {
		return new MusicServiceBinder<MusicServiceCallback>(this);
	}

	@Override
	public void onSongChanged(Uri songUri) {
		if (songUri == null) {
			updateNowPlayingOnUIThread(null, null, null);
			return;
		}
		
		ContentResolver resolver = getContentResolver();
		String[] projection = new String[] { Songs.SONG_ALBUM_ID,
				Songs.SONG_TITLE, Songs.SONG_ARTIST};
		Cursor cursor = null;
		try {
			cursor = resolver.query(songUri, projection, null, null, null);
			if (cursor != null && cursor.moveToFirst()) {
				String title = cursor.getString(cursor.getColumnIndex(Songs.SONG_TITLE));
				String artist = cursor.getString(cursor.getColumnIndex(Songs.SONG_ARTIST));
				String albumId = cursor.getString(cursor.getColumnIndex(Songs.SONG_ALBUM_ID));
				updateNowPlayingOnUIThread(title, artist, albumId);
			} else {
				updateNowPlayingOnUIThread(null, null, null);
			}
		} catch (Exception e) {
			if (cursor != null) cursor.close();
		}
	}

	public void updateNowPlayingOnUIThread(final String title, final String artist,
			final String albumId) {
		mHandler.postDelayed(new Runnable() {
			@Override
			public void run() {
				if (mNowPlaying == null) return;
				mNowPlaying.bindView(title, artist, albumId, mImageDownloader);
			}
		}, 500);
		// It seems options menu is asynchronously created after activity has been created.
		// Initial 'now playing' state in UI is not critical, but this delay should generally
		// cause the condition mNowPlaying != null to hold.
	}
	
	@Override
	public void onPlayerStateChanged(PlayerState state) {
		// Do nothing.
	}
	
	@Override
	public void onSongDownload(Uri song, int percent) {
		// Do nothing.
	}

	@Override
	public void onSongProgress(Uri song, int percent) {
		// Do nothing.
	}
}
