/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import android.content.Context;
import android.database.Cursor;
import android.os.Bundle;
import android.support.v4.app.LoaderManager;
import android.support.v4.app.LoaderManager.LoaderCallbacks;
import android.support.v4.content.Loader;
import android.support.v4.widget.CursorAdapter;
import android.util.Log;

/**
 * @param <V>
 *            Adapter type.
 */
public abstract class LoadingFragment<V extends CursorAdapter> extends
		ListLoadingFragment implements LoaderCallbacks<Cursor>
{
	private static final String TAG = LoadingFragment.class.getSimpleName();

	protected V mAdapter;

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		mAdapter = onCreateAdapter(getSherlockActivity(), null);
		mListView.setAdapter(mAdapter);
		registerForContextMenu(mListView);
		
		LoaderManager lm = getSherlockActivity().getSupportLoaderManager();
		lm.initLoader(getLoaderId(), null, this);
	}

	@Override
	public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
		Log.d(TAG, "onLoadFinished()");
		show(cursor);
		setIsLoading(false);
	}

	@Override
	public void onLoaderReset(Loader<Cursor> loader) {
		Log.d(TAG, "onLoaderReset()");
		mAdapter.swapCursor(null);
		setIsLoading(false);
	}

	public abstract V onCreateAdapter(Context context, Cursor cursor);

	public void show(Cursor cursor) {
		mAdapter.swapCursor(cursor);
	}
}
