/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012-2013 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.AccountUtils.AUTH_TOKEN_TYPE;
import static com.ubuntuone.android.music.util.AccountUtils.OLD_U1_ACCOUNT_TYPE;
import static com.ubuntuone.android.music.util.AccountUtils.U1F_PACKAGE;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.io.IOException;

import org.apache.http.client.HttpClient;

import android.accounts.Account;
import android.accounts.AccountManager;
import android.accounts.AccountManagerCallback;
import android.accounts.AccountManagerFuture;
import android.accounts.AuthenticatorException;
import android.accounts.OperationCanceledException;
import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.TargetApi;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.android.music.util.AccountUtils;
import com.ubuntuone.android.music.util.HttpManager;
import com.ubuntuone.api.sso.U1AuthAPI;
import com.ubuntuone.api.sso.authorizer.Authorizer;
import com.ubuntuone.api.sso.authorizer.BasicAuthorizer;
import com.ubuntuone.api.sso.authorizer.OAuthAuthorizer;
import com.ubuntuone.api.sso.exceptions.AuthenticationException;
import com.ubuntuone.api.sso.exceptions.U1PingException;
import com.ubuntuone.api.sso.model.AuthenticateResponse;

public class LoginActivity extends Activity {
	private static final String TAG = makeLogTag(LoginActivity.class);
	
	public static final String EXTRA_EMAIL = "com.ubuntu.one.android.music.extra.EMAIL";
	
	public static final int DIALOG_ACCOUNTS = 0;
	
	public static final int REQUEST_SIGN_IN = 0;
	public static final int REQUEST_AM_SIGN_IN = 1;

	private AccountManager mAccountManager;

	private UserSignInTask mAuthTask = null;

	private String mEmail;
	private String mPassword;
	private String mAccountType;

	private EditText mEmailView;
	private EditText mPasswordView;
	private View mLoginFormView;
	private View mLoginStatusView;
	private TextView mLoginStatusMessageView;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_login);

		mEmail = getIntent().getStringExtra(EXTRA_EMAIL);
		mEmailView = (EditText) findViewById(R.id.email);
		mEmailView.setText(mEmail);

		mPasswordView = (EditText) findViewById(R.id.password);
		mPasswordView
				.setOnEditorActionListener(new TextView.OnEditorActionListener() {
					@Override
					public boolean onEditorAction(TextView textView, int id,
							KeyEvent keyEvent) {
						if (id == R.id.login || id == EditorInfo.IME_NULL) {
							attemptLogin();
							return true;
						}
						return false;
					}
				});

		mLoginFormView = findViewById(R.id.login_form);
		mLoginStatusView = findViewById(R.id.login_status);
		mLoginStatusMessageView = (TextView) findViewById(R.id.login_status_message);

		findViewById(R.id.sign_in_button).setOnClickListener(
				new View.OnClickListener() {
					@Override
					public void onClick(View view) {
						attemptLogin();
					}
				});

		mAccountManager = AccountManager.get(this);
		getAuthToken(mAccountManager);
	}
	
	private void getAuthToken(AccountManager accountManager) {
		mAccountType = AccountUtils.getPreferredAccountType(this);
		Account[] accounts = accountManager.getAccountsByType(mAccountType);
		if (accounts.length == 0 && isUbuntuOneFilesInstalled()) {
			/*
			 * Prefer Ubuntu One Files authenticator over simple form sign in. The side effect
			 * is creation of an Ubuntu One account in AccountManager, which other Ubuntu One
			 * related apps can leverage in the future for single-touch sign in flow.
			 */
			createAccountWithUbuntuOneFiles();
		} else if (accounts.length == 1) {
			gotAccount(accounts[0]);
		} else if (accounts.length > 1) {
			showDialog(DIALOG_ACCOUNTS);
		}
	}
	
	private boolean isUbuntuOneFilesInstalled() {
		try {
			// Not interested in return value.
			getPackageManager().getApplicationInfo(U1F_PACKAGE, 0);
			return true;
		} catch (PackageManager.NameNotFoundException e) {
			return false;
		}
	}
	
	private void createAccountWithUbuntuOneFiles() {
		try {
			Intent intent = new Intent(AccountUtils.ACTION_SIGN_IN);
			startActivityForResult(intent, 0);
		} catch (SecurityException e) {
			// TODO karni: LoginActivity from U1F needs to be exported.
			// Ignored, will remove the try wrapper once above fix released.
		}
	}
	
	@SuppressWarnings("unused")
	private void createAccountWithAccountManager() {
		// Alternative sign in flow which would require android.permission.MANAGE_ACCOUNTS
		String[] features = new String[]{};
		Bundle options = new Bundle();
		mAccountManager.addAccount(OLD_U1_ACCOUNT_TYPE, AUTH_TOKEN_TYPE, features, options,
				this, new OnTokenAcquired(), new Handler(new OnError()));
	}

	@Override
	protected Dialog onCreateDialog(int id) {
		switch (id) {
		case DIALOG_ACCOUNTS:
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setTitle(R.string.select_account);

			final Account[] accounts = mAccountManager.getAccountsByType(mAccountType);
			final String[] names = getUbuntuOneAccountNames(accounts);

			builder.setItems(names, new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int which) {
					gotAccount(accounts[which]);
				}
			});
			return builder.create();
		}
		return null;
	}

	private String[] getUbuntuOneAccountNames(Account[] accounts) {
		final int size = accounts.length;
		String[] names = new String[size];
		for (int i = 0; i < size; i++) {
			names[i] = accounts[i].name;
		}
		return names;
	}

	private void gotAccount(Account account) {
		Bundle options = new Bundle();
		mAccountManager.getAuthToken(account, AUTH_TOKEN_TYPE, options, this,
				new OnTokenAcquired(), new Handler(new OnError()));
	}

	private class OnTokenAcquired implements AccountManagerCallback<Bundle> {
		@Override
		public void run(AccountManagerFuture<Bundle> result) {
			try {
				Context context = getApplicationContext();
				Bundle bundle = result.getResult();
				
				String authToken = bundle.getString(AccountManager.KEY_AUTHTOKEN);
				Intent intent = (Intent) bundle.get(AccountManager.KEY_INTENT);
				
				if (authToken != null) {
					Log.i(TAG, "Got access token from AccountManager.");
					AccountUtils.setAuthToken(context, authToken);
					String accountName = bundle.getString(AccountManager.KEY_ACCOUNT_NAME);
					if (accountName == null) {
						Log.w(TAG, "authToken received, but accout name is missing.");
						return;
					}
					AccountUtils.setChosenAccountName(context, accountName);
					setResult(RESULT_OK);
					finish();
				} else if (intent != null) {
					Log.i(TAG, "Got intent from AccountManager. Launching activity.");
					startActivityForResult(intent, REQUEST_AM_SIGN_IN);
				} else {
					getAuthToken(mAccountManager);
				}
			} catch (OperationCanceledException e) {
				showErrorToast(getString(R.string.toast_canceled), Toast.LENGTH_SHORT);
			} catch (AuthenticatorException e) {
				String toastText = getString(R.string.sign_in_failed);
				String error = e.getMessage();
				if (error != null) {
					toastText += ": " + error;
				}
				showErrorToast(toastText, Toast.LENGTH_LONG);
			} catch (IOException e) {
				showErrorToast(getString(R.string.network_problem), Toast.LENGTH_LONG);
			}
		}
	}
	
	private void showErrorToast(final String text, final int length) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				Toast.makeText(getBaseContext(), text, length).show();
			}
		});
	}
	
	private class OnError implements Handler.Callback {
		@Override
		public boolean handleMessage(Message msg) {
			// Our authenticator provides no authenticator error callbacks.
			return false;
		}
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		Log.i(TAG, (resultCode == RESULT_OK) ? "Auth result success!" : "Auth result canceled.");
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		// TODO karni: Add "Forgot password" menu option.
		return true;
	}

	public void attemptLogin() {
		if (mAuthTask != null) {
			return;
		}

		mEmailView.setError(null);
		mPasswordView.setError(null);

		mEmail = mEmailView.getText().toString();
		mPassword = mPasswordView.getText().toString();

		boolean cancel = false;
		View focusView = null;

		if (TextUtils.isEmpty(mPassword)) {
			mPasswordView.setError(getString(R.string.error_field_required));
			focusView = mPasswordView;
			cancel = true;
		}

		if (TextUtils.isEmpty(mEmail)) {
			mEmailView.setError(getString(R.string.error_field_required));
			focusView = mEmailView;
			cancel = true;
		} else if (!mEmail.contains("@")) {
			mEmailView.setError(getString(R.string.error_invalid_email));
			focusView = mEmailView;
			cancel = true;
		}

		if (cancel) {
			focusView.requestFocus();
		} else {
			mLoginStatusMessageView.setText(R.string.login_progress_signing_in);
			showProgress(true);
			mAuthTask = new UserSignInTask();
			mAuthTask.execute();
		}
	}

	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
	private void showProgress(final boolean show) {
		// On Honeycomb MR2 we have the ViewPropertyAnimator APIs, which allow
		// for very easy animations. If available, use these APIs to fade-in
		// the progress spinner.
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR2) {
			int shortAnimTime = getResources().getInteger(
					android.R.integer.config_shortAnimTime);

			mLoginStatusView.setVisibility(View.VISIBLE);
			mLoginStatusView.animate().setDuration(shortAnimTime)
					.alpha(show ? 1 : 0)
					.setListener(new AnimatorListenerAdapter() {
						@Override
						public void onAnimationEnd(Animator animation) {
							mLoginStatusView.setVisibility(show ? View.VISIBLE
									: View.GONE);
						}
					});

			mLoginFormView.setVisibility(View.VISIBLE);
			mLoginFormView.animate().setDuration(shortAnimTime)
					.alpha(show ? 0 : 1)
					.setListener(new AnimatorListenerAdapter() {
						@Override
						public void onAnimationEnd(Animator animation) {
							mLoginFormView.setVisibility(show ? View.GONE
									: View.VISIBLE);
						}
					});
		} else {
			// The ViewPropertyAnimator APIs are not available, so simply show
			// and hide the relevant UI components.
			mLoginStatusView.setVisibility(show ? View.VISIBLE : View.GONE);
			mLoginFormView.setVisibility(show ? View.GONE : View.VISIBLE);
		}
	}

	public class UserSignInTask extends AsyncTask<Void, Void, Boolean> {
		private int errorResId = 0;
		
		@Override
		protected Boolean doInBackground(Void... params) {
			UbuntuOneMusic app = (UbuntuOneMusic) getApplication();
			final String appPackageName = app.getPackageName();
			final String appVersion = app.getApplicationVersion();
			
			HttpClient httpClient = HttpManager.getClient();
			Authorizer authorizer = new BasicAuthorizer(mEmail, mPassword);
			
			U1AuthAPI api = new U1AuthAPI(appPackageName, appVersion, httpClient, authorizer);
			
			final String tokenName = String.format("Ubuntu One Music @ %s", Build.MODEL);
			try {
				AuthenticateResponse authenticateResponse = api.authenticate(tokenName);
				String authToken = authenticateResponse.getSerialized();
				
				authorizer = new OAuthAuthorizer(authenticateResponse);
				api.setAuthorizer(authorizer);
				
				setupUbuntuOne(api, tokenName);
				
				Log.i(TAG, "Auth token received: " + ((authToken != null) ? "yes" : "no"));
				AccountUtils.setAuthToken(app, authToken);
				AccountUtils.setChosenAccountName(app, AccountUtils.NOT_AM_ACCOUNT);
				return true;
			} catch (AuthenticationException e) {
				Log.e(TAG, "AuthenticationException", e);
				errorResId = R.string.toast_incorrect_credentials;
			} catch (IOException e) {
				Log.e(TAG, "IOException", e);
				errorResId = R.string.toast_connectivity_problem;
			} catch (U1PingException e) {
				Log.e(TAG, "U1PingException", e);
				errorResId = R.string.toast_please_try_again_later;
			}
			return false;
		}
		
		private void setupUbuntuOne(U1AuthAPI api, String tokenName) throws U1PingException {
			int tries = 1;
			try {
				api.pingUbuntuOne(tokenName);
				Log.i(TAG, "Ubunut One ready to rock.");
			} catch (U1PingException e) {
				Log.w(TAG, String.format("Attempt #%d to ping Ubuntu One failed.", tries));
				tries++;
				if (tries == 4) {
					throw e;
				}
			}
		}

		@Override
		protected void onPostExecute(final Boolean success) {
			mAuthTask = null;
			showProgress(false);

			if (success) {
				setResult(RESULT_OK);
				finish();
			} else if (errorResId != 0) {
				Toast.makeText(getApplicationContext(), getString(errorResId),
						Toast.LENGTH_LONG).show();
			}
		}

		@Override
		protected void onCancelled() {
			mAuthTask = null;
			showProgress(false);
		}
	}
}
