/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.widget.Toast;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicProviderUtils;

public class PlayerActivity extends SherlockFragmentActivity
{
	public static final String EXTRA_SONG = "song";
	public static final String EXTRA_AUTO_PLAY = "auto_play";
	public static final String EXTRA_SHUFFLE_PLAY = "shuffle_play";
	public static final String EXTRA_REPEAT_PLAY = "repeat_play";
	public static final String EXTRA_SHOW_QUEUE = "show_queue";
	
	/* package */ boolean _reshuffleQueueTaskLaunched = false;
	private ReshuffleQueueSongsTask mReshuffleQueueSongsTask;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_player);

		ActionBar ab = getSupportActionBar();
		ab.setTitle(R.string.now_playing);
		ab.setDisplayHomeAsUpEnabled(true);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// Should start intent for home / artist albums?
			finish();
			return true;

		default:
			return super.onOptionsItemSelected(item);
		}
	}
	
	public static void startWithQueue(Context context) {
		Intent intent = new Intent(context, PlayerActivity.class);
		intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		intent.putExtra(PlayerActivity.EXTRA_SHOW_QUEUE, true);
		context.startActivity(intent);
	}
	
	public static void startFrom(Context context, boolean autoPlay) {
		Intent intent = new Intent(context, PlayerActivity.class);
		intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		if (autoPlay) {
			intent.putExtra(PlayerActivity.EXTRA_AUTO_PLAY, true);
		}
		context.startActivity(intent);
	}

	public static void startFrom(Context context, Uri songUri) {
		Intent intent = new Intent(context, PlayerActivity.class);
		intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		if (songUri != null) {
			intent.putExtra(PlayerActivity.EXTRA_SONG, songUri);
		}
		context.startActivity(intent);
	}
	
	public static void startFrom(Context context, Boolean shuffle, Boolean repeat) {
		Intent intent = new Intent(context, PlayerActivity.class);
		intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
		if (shuffle != null) {
			intent.putExtra(PlayerActivity.EXTRA_SHUFFLE_PLAY, shuffle);
		}
		if (repeat != null) {
			intent.putExtra(PlayerActivity.EXTRA_REPEAT_PLAY, repeat);
		}
		intent.putExtra(PlayerActivity.EXTRA_AUTO_PLAY, true);
		context.startActivity(intent);
	}
	
	public void reshuffleQueueSongs() {
		if (mReshuffleQueueSongsTask == null) {
			mReshuffleQueueSongsTask = new ReshuffleQueueSongsTask();
			mReshuffleQueueSongsTask.execute();
			_reshuffleQueueTaskLaunched = true;
		}
	}
	
	private class ReshuffleQueueSongsTask extends AsyncTask<Void, Void, Void> {
		@Override
		protected void onPreExecute() {
			if (!isFinishing()) {
				Toast.makeText(PlayerActivity.this,
						"Reshuffling...", Toast.LENGTH_SHORT).show();
			}
		}

		@Override
		protected Void doInBackground(Void... params) {
			if (!isCancelled()) {
				MusicProviderUtils.reshufflePlayQueue(PlayerActivity.this);
				MusicProviderUtils.resetSongWasPlayedFlags(PlayerActivity.this);
			}
			return null;
		}
		
		@Override
		protected void onPostExecute(Void result) {
			mReshuffleQueueSongsTask = null;
		}
	}
}
