/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import com.actionbarsherlock.app.ActionBar;
import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicProviderUtils;

public class PlaylistActivity extends SherlockFragmentActivity implements OnClickListener
{
	private static final String TAG = makeLogTag(PlaylistActivity.class);
	
	public static final String EXTRA_PLAYLIST_ID = Playlists.PLAYLIST_ID;
	
	private String mPlaylistId;
	private ImageView mPlaylistArt;
	private TextView mPlaylistName;
	private TextView mPlaylistSize;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_collection);
		
		mPlaylistArt = (ImageView) findViewById(R.id.image);
		mPlaylistName = (TextView) findViewById(R.id.text1);
		mPlaylistSize = (TextView) findViewById(R.id.text2);
		
		Button shuffleAllButton = (Button) findViewById(R.id.shuffle_all);
		shuffleAllButton.setOnClickListener(this);
		Button repeatAllButton = (Button) findViewById(R.id.repeat_all);
		repeatAllButton.setOnClickListener(this);

		mPlaylistId = getIntent().getStringExtra(Playlists.PLAYLIST_ID);
		if (mPlaylistId == null) {
			throw new IllegalStateException("playlist_id cannot be null");
		}

		LoaderManager lm = getSupportLoaderManager();
		Loader<?> loader = lm.getLoader(R.id.playlist_songs_loader_id);
		if (loader != null) {
			loader.stopLoading();
		}
		mPlaylistArt.setImageResource(R.drawable.default_art);

		// TODO Recycle fragment?
		SongsFragment playlistSongs = new SongsFragment();

		Intent intent = getIntent();
		String playlistName = intent.getStringExtra(Playlists.PLAYLIST_NAME);
		mPlaylistName.setText(playlistName);
		int playlistLength = intent.getIntExtra(Playlists.PLAYLIST_SONG_COUNT, 0);
		mPlaylistSize.setText(getString(R.string.playlist_stats, playlistLength));
		
		Bundle args = new Bundle();
		args.putString(Playlists.PLAYLIST_ID, mPlaylistId);
		args.putString(Playlists.PLAYLIST_NAME, playlistName);
		playlistSongs.setArguments(args);

		ActionBar ab = getSupportActionBar();
		ab.setTitle(playlistName);
		ab.setDisplayHomeAsUpEnabled(true);

		FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
		ft.replace(R.id.items, playlistSongs);
		ft.setTransition(FragmentTransaction.TRANSIT_FRAGMENT_FADE);
		ft.commit();
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// Should start intent for home / artist albums?
			finish();
			return true;

		default:
			return super.onOptionsItemSelected(item);
		}
	}
	
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.shuffle_all:
			shuffleAll(v);
			break;
		case R.id.repeat_all:
			repeatAll(v);
			break;
		}
	}
	
	public void shuffleAll(View v) {
		Uri playlistUri = Playlists.buildPlaylistUri(mPlaylistId);
		MusicProviderUtils.clearAndEnqueuePlaylist(this, playlistUri);
		PlayerActivity.startFrom(this, Boolean.TRUE, Boolean.FALSE);
	}
	
	public void repeatAll(View v) {
		Uri playlistUri = Playlists.buildPlaylistUri(mPlaylistId);
		MusicProviderUtils.clearAndEnqueuePlaylist(this, playlistUri);
		PlayerActivity.startFrom(this, Boolean.FALSE, Boolean.TRUE);
	}
	
	public static void startFrom(Context context, Cursor cursor) {
		if (cursor == null) {
			Log.w(TAG, "null cursor in PlaylistActivity.startFrom()");
			return;
		}
		
		String playlistId = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_ID));
		String title = cursor.getString(cursor.getColumnIndex(Playlists.PLAYLIST_NAME));
		int songCount = cursor.getInt(cursor.getColumnIndex(Playlists.PLAYLIST_SONG_COUNT));
		
		startFrom(context, playlistId, title, songCount);
	}
	
	public static void startFrom(Context context, String playlistId, String title, int songCount) {
		if (playlistId == null) {
			throw new IllegalArgumentException("PlaylistActivity: playlistId must not be null");
		}
		
		Intent intent = new Intent(context, PlaylistActivity.class);
		intent.putExtra(Playlists.PLAYLIST_ID, playlistId);
		intent.putExtra(Playlists.PLAYLIST_NAME, title);
		intent.putExtra(Playlists.PLAYLIST_SONG_COUNT, songCount);
		
		Log.d(TAG, "Launching PlaylistActivity for playlistId " + playlistId);
		context.startActivity(intent);
	}
}
