/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.ui.HomeActivity.ERROR_DIALOG_ID;
import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;

import java.util.ArrayList;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.Context;
import android.database.Cursor;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Adapter;
import android.widget.Button;
import android.widget.ListView;

import com.actionbarsherlock.app.SherlockFragmentActivity;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.UbuntuOneMusic;
import com.ubuntuone.android.music.adapter.U1PlaylistAdapter;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.provider.dao.PlaylistDao;
import com.ubuntuone.android.music.ui.dialog.InfoAlertDialog;
import com.ubuntuone.android.music.ui.dialog.InputAlertDialog;
import com.ubuntuone.android.music.ui.dialog.InputAlertDialog.InputAlertDialogListener;
import com.ubuntuone.android.music.ui.dialog.ModalAlertDialog;
import com.ubuntuone.android.music.util.AccountUtils;
import com.ubuntuone.android.music.util.NotAuthenticatedException;
import com.ubuntuone.api.music.U1MusicAPI;
import com.ubuntuone.api.music.client.Failure;
import com.ubuntuone.api.music.model.U1Playlist;
import com.ubuntuone.api.music.request.U1PlaylistCreateListener;
import com.ubuntuone.api.music.request.U1PlaylistListener;

public class PlaylistsFragment extends LoadingFragment<U1PlaylistAdapter> implements
		OnClickListener, InputAlertDialogListener
{
	private static final String TAG = makeLogTag(PlaylistsFragment.class);
	
	public static final String EXTRA_INCLUDE_CUSTOM = "include_custom";
	
	private boolean mIncludeCustom = false;
	
	private OnPlaylistSelectedListener mOnPlaylistSelectedListener;

	@Override
	public int onGetHeaderResId() {
		return R.string.playlists_title;
	}
	
	@Override
	public int onGetLoaderId() {
		return R.id.playlist_loader_id;
	}

	@Override
	public int onGetFragmentLayoutResId() {
		return R.layout.fragment_playlists;
	}

	@Override
	public int onGetEmptyListResId() {
		return R.string.playlists_no_found;
	}

	@Override
	public int onGetLoadingListResId() {
		return R.string.playlists_loading;
	}
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		Bundle args = getArguments();
		if (args != null) {
			mIncludeCustom = args.getBoolean(EXTRA_INCLUDE_CUSTOM, false);
		}
		
		View view = super.onCreateView(inflater, container, savedInstanceState);
		
		Button createPlaylist = (Button) view.findViewById(R.id.create_playlist);
		createPlaylist.setOnClickListener(this);
		
		try {
			mOnPlaylistSelectedListener = (OnPlaylistSelectedListener) getActivity();
		} catch (ClassCastException e) {
			throw new IllegalArgumentException("Activity must implement OnPlaylistSelectedListener");
		}
		
		return view;
	}
	
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		mOnPlaylistSelectedListener.onPlaylistSelected(mAdapter, position, mIncludeCustom);
	}

	@Override
	public U1PlaylistAdapter onCreateAdapter(Context context, Cursor cursor) {
		Log.d(TAG, "PlaylistsFragment#onCreateAdapter");
		
		return new U1PlaylistAdapter(context, cursor, mIncludeCustom);
	}

	@Override
	public Loader<Cursor> onCreateLoader(int id, Bundle args) {
		Log.d(TAG, "PlaylistsFragment#onCreateLoader");
		
		String selection = Playlists.PLAYLIST_ID + "!=?";
		String[] selectionArgs = new String[] { MusicProviderUtils.QUEUE_ID };
		CursorLoader loader = new CursorLoader(getSherlockActivity(), Playlists.CONTENT_URI,
				Playlists.DEFAULT_PROJECTION, selection, selectionArgs, Playlists.DEFAULT_SORT);
		setIsLoading(true);
		return loader;
	}

	@Override
	public void onClick(View v) {
		if (v.getId() == R.id.create_playlist) {
			SherlockFragmentActivity activity = getSherlockActivity();
			if (activity != null) {
				FragmentTransaction ft = activity.getSupportFragmentManager().beginTransaction();
				Fragment prev = activity.getSupportFragmentManager()
						.findFragmentByTag("createPlaylistDialog");
				if (prev != null) {
					ft.remove(prev);
				}
				ft.addToBackStack(null);
				
				DialogFragment frag = InputAlertDialog.newInstance(
						HomeActivity.CREATE_PLAYLIST_DIALOG_ID, R.string.playlist_title);
				frag.show(ft, "createPlaylistDialog");
			}
		}
	}

	@Override
	public void onPositiveButtonClicked(int dialogId, String text) {
		CreatePlaylistTask createPlaylistTask = new CreatePlaylistTask();
		createPlaylistTask.execute(text);
	}

	@Override
	public void onNegativeButtonClicked(int dialogId) {
		// Nothing to do.
	}
	
	private class CreatePlaylistTask extends AsyncTask<String, Void, Boolean> {
		private DialogFragment mDialogFragment;
		private String mPlaylistId = null;
		private String mErrorMessage;
		
		@Override
		protected void onPreExecute() {
			SherlockFragmentActivity activity = getSherlockActivity();
			if (activity != null) {
				mDialogFragment = ModalAlertDialog.newInstance();
				mDialogFragment.show(activity.getSupportFragmentManager(), "pleaseWaitDialog");
			}
		}

		@Override
		protected Boolean doInBackground(String... params) {
			if (params.length != 1) {
				throw new IllegalArgumentException(
						"CreatePlaylistTask takes one argument, the new playlist name");
			}
			final String name = params[0];
			
			U1MusicAPI api;
			try {
				api = UbuntuOneMusic.getInstance().getMusicApi();
			} catch (NotAuthenticatedException e) {
				AccountUtils.startAuthenticationFlow(getActivity(), 0);
				return false;
			}
			ArrayList<String> emptyList = new ArrayList<String>();
			api.createPlaylist(name, emptyList, new U1PlaylistCreateListener() {
				@Override
				public void onSuccess(String playlistId) {
					// TODO Persist playlist once CSRF fix is rolled out.
					Log.i(TAG, "Created playlist with id: " + playlistId);
					mPlaylistId = playlistId;
				}

				@Override
				public void onFailure(Failure failure) {
					mPlaylistId = null;
					mErrorMessage = failure.getMessage();
					Log.e(TAG, "Failed to create playlist: " + failure.getOopsId());
				}
			});
			
			// TODO This block will be removed soon, see TODO above.
			Activity activity = getSherlockActivity();
			final ContentResolver resolver = activity != null ? activity.getContentResolver() : null;
			if (mPlaylistId != null && resolver != null) {
				api.getPlaylists(new U1PlaylistListener() {
					@Override
					public void onSuccess(U1Playlist playlist) {
						if (mPlaylistId.equals(playlist.getId())) {
							new PlaylistDao().updateOrInsert(resolver, playlist);
						}
					}
				});
			}
			
			return mPlaylistId != null;
		}

		@Override
		protected void onPostExecute(Boolean created) {
			if (mDialogFragment != null && mDialogFragment.isAdded()) {
				mDialogFragment.dismiss();
			}
			
			SherlockFragmentActivity activity = getSherlockActivity();
			if (created && activity != null) {
				((HomeActivity) activity).forcePlaylistsRefresh();
			} else if (!created  && activity != null && !TextUtils.isEmpty(mErrorMessage)) {
				DialogFragment frag = InfoAlertDialog.newInstance(ERROR_DIALOG_ID, 0, mErrorMessage);
				frag.show(activity.getSupportFragmentManager(), null);
			}
		}
	}
	
	public interface OnPlaylistSelectedListener {
		public void onPlaylistSelected(Adapter adapter, int position, boolean hasHeaders);
	}
}
