/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012-2013 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui;

import static com.ubuntuone.android.music.util.LogUtils.makeLogTag;
import static com.ubuntuone.android.music.util.StorageUtils.MiB;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;

import android.annotation.TargetApi;
import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.PreferenceActivity;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.util.Log;

import com.ubuntuone.android.music.Constants;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.util.SeekBarPreference;
import com.ubuntuone.android.music.util.StorageUtils;
import com.ubuntuone.android.music.util.StorageUtils.InvalidStorage;
import com.ubuntuone.android.music.util.UIUtils;
 
public class PreferencesActivity extends PreferenceActivity {
	private static final String TAG = makeLogTag(PreferencesActivity.class.getSimpleName());
	
	protected Method mLoadHeaders = null;
	protected Method mHasHeaders = null;
	protected boolean mIsTablet;

	public boolean hasHeadersCompat() {
		if (mHasHeaders != null && mLoadHeaders != null) {
			try {
				return (Boolean) mHasHeaders.invoke(this);
			} catch (IllegalArgumentException e) {
			} catch (IllegalAccessException e) {
			} catch (InvocationTargetException e) {
			}
		}
		return false;
	}

	@SuppressWarnings("deprecation")
	@Override
	public void onCreate(Bundle aSavedState) {
		// onBuildHeaders() will be called during super.onCreate()
		try {
			mLoadHeaders = getClass().getMethod("loadHeadersFromResource", int.class, List.class);
			mHasHeaders = getClass().getMethod("hasHeaders");
		} catch (NoSuchMethodException e) {
			// We're handling an old API.
		}
		mIsTablet = UIUtils.isTablet(this);
		super.onCreate(aSavedState);
		if (!hasHeadersCompat() || !mIsTablet) {
			addPreferencesFromResource(R.xml.preferences_account);
			addPreferencesFromResource(R.xml.preferences_cache);
		}
		
		setupCacheLocationPreference();
		setupCacheSizePreference(this);
	}

	@Override
	public void onBuildHeaders(List<Header> target) {
		try {
			if (mIsTablet) {
				mLoadHeaders.invoke(this, new Object[] { R.xml.preference_headers, target });
			}
		} catch (IllegalArgumentException e) {
		} catch (IllegalAccessException e) {
		} catch (InvocationTargetException e) {
		}
	}

	@TargetApi(Build.VERSION_CODES.HONEYCOMB)
	static public class PreferencesFragment extends PreferenceFragment {
		@Override
		public void onCreate(Bundle savedInstanceState) {
			super.onCreate(savedInstanceState);
			Context context = getActivity().getApplicationContext();
			int preferencesResource = context.getResources().getIdentifier(
					getArguments().getString("preferenceResource"), "xml",
					context.getPackageName());
			PreferenceManager.setDefaultValues(context, preferencesResource, false);
			addPreferencesFromResource(preferencesResource);
			// Hack. On tablets, hide the category title duplicated in fragment title.
			findPreference("preferenceCategory").setTitle("");
		}
	}
	
	public void setupCacheLocationPreference() {
		@SuppressWarnings("deprecation")
		final ListPreference cacheLocation = (ListPreference) findPreference(
				Constants.Preference.CACHE_LOCATION_KEY);
		final String[] storagePaths = StorageUtils.getAvailableStoragePaths();
		Log.i(TAG, "Available storage paths: " + Arrays.toString(storagePaths));
		if (storagePaths != null) {
			cacheLocation.setEntries(storagePaths);
			cacheLocation.setEntryValues(storagePaths);
			
			String storage = cacheLocation.getValue();
			cacheLocation.setValue(storage);
			cacheLocation.setSummary(storage);
			try {
				int index = StorageUtils.getSelectedStorageIndex(storage, storagePaths);
				cacheLocation.setValueIndex(index);
			} catch (InvalidStorage e) {
				// Should not happen, as the value has been already set.
			}
			
			cacheLocation.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
				@Override
				public boolean onPreferenceChange(Preference preference, Object newValue) {
					// TODO karni: Move cached music to new location or at least notify the user.
					String location = (String) newValue;
					cacheLocation.setSummary(location);
					return true;
				}
			});
		} else {
			cacheLocation.setEnabled(false);
		}
	}
	
	public void setupCacheSizePreference(Context context) {
		@SuppressWarnings("deprecation")
		SeekBarPreference seekBarPreference = (SeekBarPreference) findPreference(
				Constants.Preference.CACHE_SIZE_KEY);
		String storage = StorageUtils.getMusicDirectory(context);
		
		long cacheSize = 0; // TODO karni: Calculate space used by cache.
		long min = Math.max(100 * MiB, cacheSize);
		long max = StorageUtils.getAvailableStorage(context, storage) - cacheSize;
		
		seekBarPreference.setMinValue((int)(min / MiB));
		seekBarPreference.setMaxValue((int)(max / MiB));
		seekBarPreference.setDefaultValue((int)(max / MiB));
	}
}
