/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui.dialog;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;
import android.view.Gravity;
import android.view.ViewGroup.LayoutParams;
import android.widget.TextView;

import com.actionbarsherlock.app.SherlockDialogFragment;
import com.ubuntuone.android.music.R;

public class InfoAlertDialog extends SherlockDialogFragment
{
	private InfoAlertDialogListener mListener;
	
	public static InfoAlertDialog newInstance(int dialogId, int titleId, String message) {
		InfoAlertDialog frag = new InfoAlertDialog();
		Bundle args = new Bundle();
		args.putInt("dialogId", dialogId);
		if (titleId != 0) args.putInt("titleId", titleId);
		args.putString("message", message);
		frag.setArguments(args);
		return frag;
	}
	
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		try {
			mListener = (InfoAlertDialogListener) getSherlockActivity();
		} catch (ClassCastException e) {
			// Ignore. Caller may be not interested in dismissing the dialog.
		}
	}
	
	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState) {
		Bundle args = getArguments();
		final int dialogId = args.getInt("dialogId");
		final String message = args.getString("message");
		
		final TextView view = new TextView(getSherlockActivity());
		view.setTextAppearance(getSherlockActivity(), android.R.style.TextAppearance_Large);
		view.setGravity(Gravity.CENTER);
		view.setLayoutParams(new LayoutParams(LayoutParams.MATCH_PARENT,LayoutParams.WRAP_CONTENT));
		view.setText(message);
		
		OnClickListener listener = new OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				switch (which) {
				case Dialog.BUTTON_POSITIVE:
					if (mListener != null) mListener.onPositiveButtonClicked(dialogId);
					break;
				}
			}
		};
		
		AlertDialog.Builder builder = new AlertDialog.Builder(getActivity())
				.setView(view)
				.setPositiveButton(R.string.ok, listener);
		if (args.containsKey("titleId")) {
			int titleId = args.getInt("titleId");
			builder.setTitle(titleId);
		}
		
		return builder.create();
	}
	
	public static interface InfoAlertDialogListener {
		public void onPositiveButtonClicked(int dialogId);
	}
}
