/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.ui.dialog;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;
import android.widget.EditText;

import com.actionbarsherlock.app.SherlockDialogFragment;
import com.ubuntuone.android.music.R;

public class InputAlertDialog extends SherlockDialogFragment
{
	private InputAlertDialogListener mListener;
	
	public static InputAlertDialog newInstance(int dialogId, int titleId) {
		InputAlertDialog frag = new InputAlertDialog();
		Bundle args = new Bundle();
		args.putInt("dialogId", dialogId);
		args.putInt("titleId", titleId);
		frag.setArguments(args);
		return frag;
	}
	
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
		try {
			mListener = (InputAlertDialogListener) getSherlockActivity();
		} catch (ClassCastException e) {
			throw new ClassCastException(activity.toString()
					+ " must implement InputAlertDialogListener");
		}
	}
	
	@Override
	public Dialog onCreateDialog(Bundle savedInstanceState) {
		final int dialogId = getArguments().getInt("dialogId");
		final int titleId = getArguments().getInt("titleId");
		
		final EditText view = new EditText(getSherlockActivity());
		
		OnClickListener listener = new OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				switch (which) {
				case Dialog.BUTTON_POSITIVE:
					mListener.onPositiveButtonClicked(dialogId, view.getText().toString());
					break;
				case Dialog.BUTTON_NEGATIVE:
					mListener.onNegativeButtonClicked(dialogId);
					break;
				}
			}
		};
		
		return new AlertDialog.Builder(getActivity())
				.setTitle(titleId)
				.setView(view)
				.setPositiveButton(R.string.ok, listener)
				.setNegativeButton(R.string.cancel, listener)
				.create();
	}
	
	public static interface InputAlertDialogListener {
		public void onPositiveButtonClicked(int dialogId, String text);
		public void onNegativeButtonClicked(int dialogId);
	}
}
