/*
 * Ubuntu One Music - stream music from Ubuntu One cloud storage.
 * 
 * Copyright 2012 Canonical Ltd.
 * 
 * This file is part of Ubuntu One Music app.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses 
 */

package com.ubuntuone.android.music.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public final class HashUtils
{
	private static final String MD5 = "MD5";
	private static final int CHUNK_SIZE = 1024 * 8;
	
	private HashUtils() {
	}
	
	public static String md5(InputStream stream) throws IOException {
		MessageDigest dataDigest;
		try {
			dataDigest = MessageDigest.getInstance(MD5);
		} catch (NoSuchAlgorithmException e) {
			throw new UnsupportedOperationException("MD5 algorithm not available.");
		}
		
		dataDigest.reset();
		byte[] data = new byte[CHUNK_SIZE];
		for (int n = 0; (n = stream.read(data)) != -1; ) {
			dataDigest.update(data, 0, n);
		}
		final byte[] digest = dataDigest.digest();
		
		return toHexString(digest);
	}
	
	public static String md5(File file) throws IOException {
		return md5(new FileInputStream(file));
	}
	
	public static String md5(String data) {
		MessageDigest dataDigest;
		try {
			dataDigest = MessageDigest.getInstance(MD5);
		} catch (NoSuchAlgorithmException e) {
			throw new UnsupportedOperationException("MD5 algorithm not available.");
		}
		
		dataDigest.reset();
		dataDigest.update(data.getBytes());
		final byte[] digest = dataDigest.digest();
		
		return toHexString(digest);
	}
	
	private static String toHexString(byte[] data) {
		final StringBuilder md5 = new StringBuilder(46);
		
		for (int i = 0; i < data.length; i++) {
			md5.append(Integer.toHexString((0xFF & data[i]) + 0x100)
					.substring(1));
		}
		return md5.toString();
	}
}
