package com.ubuntuone.android.music;

import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;

import com.ubuntuone.android.music.provider.MusicContentValues;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.music.provider.MusicDatabase.Tables;
import com.ubuntuone.android.music.util.HashUtils;
import com.ubuntuone.api.music.model.U1Playlist;

public final class MockData
{
	// Update these values when adding more mock data.
	public static int ARTISTS_COUNT = 2;
	public static int ALBUMS_COUNT = 3;
	public static int SONGS_COUNT = 3;
	public static int STARRED_SONGS_COUNT = 2;
	public static int CACHED_SONGS_COUNT = 2;
	public static int PLAYLISTS_COUNT = 1 + 2; // _queue + user playlists
	public static int PLAYLIST_SONGS_COUNT = 3;
	public static int QUEUE_SONG_COUNT = 1;
	
	public static void setUp(SQLiteDatabase db) {
		db.beginTransaction();
		
		U1Playlist queuePlaylist = new U1Playlist(
				MusicProviderUtils.QUEUE_ID, "", MusicProviderUtils.QUEUE_NAME, 0);
		ContentValues values = MusicContentValues.forPlaylist(queuePlaylist);
		db.insert(Tables.PLAYLISTS, null, values);
		
		// Insert artist(s).
		Object[] artist1Args = new Object[] {
				HashUtils.md5("artist1-artist_name"), "artist1-artist_name",
				"album1-album_id, album2-album_id", 2,
				"artist1-artist_url", "artist1-artist_art_url"
		};
		Object[] artist2Args = new Object[] {
				HashUtils.md5("artist2-artist_name"), "artist2-artist_name", "album2-album_id", 4,
				"artist2-artist_url", "artist2-artist_art_url"
		};
		final String artistInsertSql = "INSERT INTO artists(artist_id, artist_name, album_list, " +
				"song_count, artist_url, artist_art_url) VALUES (?, ?, ?, ?, ?, ?)"; 
		db.execSQL(artistInsertSql, artist1Args);
		db.execSQL(artistInsertSql, artist2Args);
		
		// Insert artist album(s).
		Object[] artist1Album1Args = new Object[] {
				HashUtils.md5("artist1-artist_name"),
				HashUtils.md5("album1-album_titleartist1-artist_name")
		};
		Object[] artist1Album2Args = new Object[] {
				HashUtils.md5("artist1-artist_name"),
				HashUtils.md5("album2-album_titleartist1-artist_name")
		};
		Object[] artist2Album1Args = new Object[] {
				HashUtils.md5("artist2-artist_name"),
				HashUtils.md5("album21-album_titleartist2-artist_name")
		};
		final String artistAlbumInsertSql = "INSERT INTO artists_albums(artist_id, album_id) " +
				"VALUES (?, ?)";
		db.execSQL(artistAlbumInsertSql, artist1Album1Args);
		db.execSQL(artistAlbumInsertSql, artist1Album2Args);
		db.execSQL(artistAlbumInsertSql, artist2Album1Args);
		
		// Insert album(s).
		Object[] album1Args = new Object[] {
				HashUtils.md5("album1-album_titleartist1-artist_name"), "album1-album_title",
				"artist1-artist_name", HashUtils.md5("artist1-artist_name"), 2001, 1300300300,
				"album1-album_url", "album1-album_art_url"
		};
		Object[] album2Args = new Object[] {
				HashUtils.md5("album2-album_titleartist1-artist_name"), "album2-album_title",
				"artist1-artist_name", HashUtils.md5("artist1-artist_name"), 2002, 1300300302,
				"album2-album_url", "album2-album_art_url"
		};
		Object[] album21Args = new Object[] {
				HashUtils.md5("album21-album_titleartist2-artist_name"), "album21-album_title",
				"artist2-artist_name", HashUtils.md5("artist2-artist_name"), 2002, 1300300302,
				"album21-album_url", "album21-album_art_url"
		};
		final String albumInsertSql = "INSERT INTO albums(album_id, album_title, album_artist," +
				"album_artist_id, album_year, album_date_added, album_url, album_art_url) " +
				"VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
		db.execSQL(albumInsertSql, album21Args);
		db.execSQL(albumInsertSql, album2Args);
		db.execSQL(albumInsertSql, album1Args);
		
		// Insert song(s).
		Object[] song1Args = new Object[] {
				"song1-song_id", "song1-song_title", "album1-album_title",
				HashUtils.md5("album1-album_titleartist1-artist_name"), "song1-song_album_artist",
				"artist1-artist_name", HashUtils.md5("artist1-artist_name"),
				"genre1", HashUtils.md5("genre1"), 2001,
				1, 1, 180 * 1, 1024 * 1024 * 1,
				128 * 1, "mp3", "audio/mpeg", null,
				1347324619016L, "song1-song_url", "song1-art_url", "song1-stream_url", 1
		};
		Object[] song2Args = new Object[] {
				"song2-song_id", "song2-song_title", "album1-album_title",
				HashUtils.md5("album1-album_titleartist1-artist_name"), "song2-song_album_artist",
				"artist1-artist_name", HashUtils.md5("artist1-artist_name"),
				"genre1", HashUtils.md5("genre1"), 2001,
				1, 2, 180 * 2, 1024 * 1024 * 2,
				128 * 2, "mp3", "audio/mpeg", "song2-song_path",
				1347300619018L, "song2-song_url", "song2-art_url", "song2-stream_url", 0
		};
		Object[] song3Args = new Object[] {
				"song3-song_id", "song3-song_title", "album21-album_title",
				HashUtils.md5("album21-album_titleartist2-artist_name"), "song3-song_album_artist",
				"artist2-artist_name", HashUtils.md5("artist2-artist_name"),
				"genre1", HashUtils.md5("genre1"), 2001,
				2, 3, 180 * 3, 1024 * 1024 * 3,
				128 * 3, "mp3", "audio/mpeg", "song3-song_path",
				1347300619018L, "song3-song_url", "song3-art_url", "song3-stream_url", 1
		};
		final String songInsertSql = "INSERT INTO songs(song_id, song_title, " +
				"song_album, song_album_id, song_album_artist, song_artist, " +
				"song_artist_id, song_genre, song_genre_ids, song_year, " +
				"song_disc_number, song_track, song_duration, song_size, " +
				"song_bit_rate, song_suffix, song_content_type, song_path, " +
				"last_played_at, song_url, song_art_url, song_stream_url, starred) " +
				"VALUES (?, ?, " + "?, ?, ?, ?, " + "?, ?, ?, ?, " +
				"?, ?, ?, ?, " + "?, ?, ?, ?, " + "?, ?, ?, ?, ?)";
		db.execSQL(songInsertSql, song3Args);
		db.execSQL(songInsertSql, song2Args);
		db.execSQL(songInsertSql, song1Args);
		
		// Insert playlist(s).
		Object[] playQueue = new Object[] {
				MusicProviderUtils.QUEUE_ID, "", MusicProviderUtils.QUEUE_NAME, 0 };
		Object[] playlist1Args = new Object[] {
				"playlist1-playlist_id", "playlist1-playlist_name", 2, "playlist1-playlist_url"
		};
		Object[] playlist2Args = new Object[] {
				"playlist2-playlist_id", "playlist2-playlist_name", 1, "playlist2-playlist_url"
		};
		final String playlistInsertSql = "INSERT INTO playlists(playlist_id, playlist_name," +
				"playlist_song_count, playlist_url) VALUES (?, ?, ?, ?)";
		db.execSQL(playlistInsertSql, playQueue);
		db.execSQL(playlistInsertSql, playlist2Args);
		db.execSQL(playlistInsertSql, playlist1Args);
		
		// Insert playlist song(s).
		final String playlistSongInsertSql = "INSERT INTO playlists_songs" +
				"(playlist_id, song_id) VALUES (?, ?)";
		
		Object[] playlistSong1Args = new Object[] {
				"playlist1-playlist_id", "song1-song_id"
		};
		Object[] playlistSong2Args = new Object[] {
				"playlist1-playlist_id", "song2-song_id"
		};
		db.execSQL(playlistSongInsertSql, playlistSong2Args);
		db.execSQL(playlistSongInsertSql, playlistSong1Args);
		
		Object[] playlist2Song2Args = new Object[] {
				"playlist2-playlist_id", "song2-song_id"
		};
		db.execSQL(playlistSongInsertSql, playlist2Song2Args);
		
		Object[] queueSong1Args = new Object[] {
				MusicProviderUtils.QUEUE_ID, "song2-song_id"
		};
		db.execSQL(playlistSongInsertSql, queueSong1Args);
		
		db.setTransactionSuccessful();
		db.endTransaction();
	}
	
	public static void tearDown(SQLiteDatabase db) {
		db.beginTransaction();
		
		db.delete(Tables.GENRES, null, null);
		db.delete(Tables.PLAYLISTS_SONGS, null, null);
		db.delete(Tables.PLAYLISTS, null, null);
		db.delete(Tables.SONGS, null, null);
		db.delete(Tables.ALBUMS, null, null);
		db.delete(Tables.ARTISTS_ALBUMS, null, null);
		db.delete(Tables.ARTISTS, null, null);
		
		db.setTransactionSuccessful();
		db.endTransaction();
	}
}
