package com.ubuntuone.android.music.provider;

import java.util.ArrayList;

import junit.framework.TestCase;
import android.content.ContentValues;

import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.util.HashUtils;
import com.ubuntuone.android.music.util.Lists;
import com.ubuntuone.api.music.model.U1Album;
import com.ubuntuone.api.music.model.U1Artist;
import com.ubuntuone.api.music.model.U1Playlist;
import com.ubuntuone.api.music.model.U1Song;

public class MusicContentValuesTest extends TestCase
{
	public void testFromU1Artist() {
		final String albumId1 = "b140a46b1c2c4185fa39bd73bc7e2441";
		final String albumId2 = "fe8086438e8f7dfc4e214b8db9bed3e6";
		final String albumListValue =
				String.format("%s,%s", albumId1, albumId2);
		
		final String artist = "artist name";
		final String id = HashUtils.md5(artist);
		final ArrayList<String> albumList = Lists.newArrayList();
		albumList.add(albumId1);
		albumList.add(albumId2);
		final Integer songCount = 1;
		final String artistUrl = "https://one.ubuntu.com/api/music/2.0" +
				"/artists/" + id + "/";
		final String artistArtUrl = "https://one.ubuntu.com/api/music/2.0" +
				"/artists/" + id + "/art/";
		
		final U1Artist u1Artist = new U1Artist(id, artistUrl, artist,
				artistArtUrl, albumList, songCount);
		final ContentValues cv = MusicContentValues.forArtist(u1Artist);
		
		assertEquals(id, cv.getAsString(Artists.ARTIST_ID));
		assertEquals(artist, cv.getAsString(Artists.ARTIST_NAME));
		assertEquals(albumListValue, cv.getAsString(Artists.ALBUM_LIST));
		assertEquals(songCount, cv.getAsInteger(Artists.SONG_COUNT));
		assertEquals(artistUrl, cv.getAsString(Artists.ARTIST_URL));
		assertEquals(artistArtUrl, cv.getAsString(Artists.ARTIST_ART_URL));
	}

	public void testFromU1Album() {
		final String title = "album title";
		final String artist = "artist name";
		final String id = HashUtils.md5(title+artist);
		final String artistId = HashUtils.md5(artist);
		final Integer year = 2011;
		final Long dateAdded = 1327937958L;
		final String albumUrl = "https://one.ubuntu.com/api/music/2.0" +
				"/albums/" + id + "/";
		final String albumArtUrl = "https://one.ubuntu.com/api/music/2.0" +
				"/albums/" + id + "/art/";
		
		final U1Album u1Album = new U1Album(id, albumUrl, artist, artistId,
				title, dateAdded, albumArtUrl, year);
		final ContentValues cv = MusicContentValues.forAlbum(u1Album);
		
		assertEquals(id, cv.getAsString(Albums.ALBUM_ID));
		assertEquals(title, cv.getAsString(Albums.ALBUM_TITLE));
		assertEquals(artist, cv.getAsString(Albums.ALBUM_ARTIST));
		assertEquals(artistId, cv.getAsString(Albums.ALBUM_ARTIST_ID));
		assertEquals(year, cv.getAsInteger(Albums.ALBUM_YEAR));
		assertEquals(dateAdded, cv.getAsLong(Albums.ALBUM_DATE_ADDED));
		assertEquals(albumUrl, cv.getAsString(Albums.ALBUM_URL));
		assertEquals(albumArtUrl, cv.getAsString(Albums.ALBUM_ART_URL));
	}

	public void testFromU1Song() {
		final String id ="i6aVaQDYSS6Pal0GbO4MAw";
		final String title = "Break The Noize & The Autobots Remix";
		final String album = "Dubstep";
		final String albumArtist = "Various";
		final String artist = "To The Stars";
		final String albumId = HashUtils.md5(title+artist);
		final String artistId = HashUtils.md5(artist);
		final String genre = "Dubstep";
		final String genreIds = MusicContentValues.getGenreId(genre);
		final Integer year = 2012;
		final Integer discNumber = 1;
		final Integer track = 9;
		final Integer duration = 215;
		final Integer size = 3398426;
		final Integer bitRate = 256;
		final String suffix = "mp3";
		final String contentType = "audio/mpeg";
		final String path = null;
		final String songUrl = "https://one.ubuntu.com" +
				"/api/music/2.0/songs/" + id + "/";
		final String songArtUrl = "https://one.ubuntu.com" +
				"/api/music/2.0/albums/" + albumId + "/art/";
		final String songStreamUrl = "https://streaming.one.ubuntu.com" +
				"/api/music/2.0/songs/" + id;
		
		final U1Song u1Song = new U1Song(id, songUrl, albumId, albumArtist,
				album, songArtUrl, genre, year, discNumber, track,
				duration, size, bitRate, artist, artistId, title, suffix,
				contentType, songStreamUrl, path);
		final ContentValues cv = MusicContentValues.forSong(u1Song);
		
		assertEquals(id, cv.getAsString(Songs.SONG_ID));
		assertEquals(title, cv.getAsString(Songs.SONG_TITLE));
		assertEquals(album, cv.getAsString(Songs.SONG_ALBUM));
		assertEquals(albumId, cv.getAsString(Songs.SONG_ALBUM_ID));
		assertEquals(albumArtist, cv.getAsString(Songs.SONG_ALBUM_ARTIST));
		assertEquals(artist, cv.getAsString(Songs.SONG_ARTIST));
		assertEquals(artistId, cv.getAsString(Songs.SONG_ARTIST_ID));
		assertEquals(genre, cv.getAsString(Songs.SONG_GENRE));
		assertEquals(genreIds, cv.getAsString(Songs.SONG_GENRE_ID));
		assertEquals(year, cv.getAsInteger(Songs.SONG_YEAR));
		assertEquals(discNumber, cv.getAsInteger(Songs.SONG_DISC_NUMBER));
		assertEquals(track, cv.getAsInteger(Songs.SONG_TRACK));
		assertEquals(duration, cv.getAsInteger(Songs.SONG_DURATION));
		assertEquals(size, cv.getAsInteger(Songs.SONG_SIZE));
		assertEquals(bitRate, cv.getAsInteger(Songs.SONG_BIT_RATE));
		assertEquals(suffix, cv.getAsString(Songs.SONG_SUFFIX));
		assertEquals(contentType, cv.getAsString(Songs.SONG_CONTENT_TYPE));
		// No Songs.SONG_LAST_PLAYED_AT in content values, so we don't
		// accidentally overwrite it when updating song cache entries.
		assertNull(cv.get(Songs.SONG_LAST_PLAYED_AT));
		assertEquals(songUrl, cv.getAsString(Songs.SONG_URL));
		assertEquals(songArtUrl, cv.getAsString(Songs.SONG_ART_URL));
		assertEquals(songStreamUrl, cv.getAsString(Songs.SONG_STREAM_URL));
	}

	public void testFromU1Playlist() {
		final String id = "bc54b75b-0632-4438-b6a3-c90b83362562";
		final String name = "Playlist name";
		final Integer songCount = 42;
		final String playlistUrl = "https://one.ubuntu.com/api/music/2.0/" +
				"playlists/" + id + "/";
		
		final U1Playlist u1Playlist = new U1Playlist(
				id, playlistUrl, name, songCount);
		final ContentValues cv = MusicContentValues.forPlaylist(u1Playlist);
		
		assertEquals(id, cv.getAsString(Playlists.PLAYLIST_ID));
		assertEquals(name, cv.getAsString(Playlists.PLAYLIST_NAME));
		assertEquals(songCount, cv.getAsInteger(Playlists.PLAYLIST_SONG_COUNT));
		assertEquals(playlistUrl, cv.getAsString(Playlists.PLAYLIST_URL));
	}
	
	public void testFromPlaylistSong() {
		final String playlistId = "bc54b75b-0632-4438-b6a3-c90b83362562";
		final String songId ="i6aVaQDYSS6Pal0GbO4MAw";
		
		final ContentValues cv = MusicContentValues.forPlaylistSong(playlistId, songId);
		
		assertEquals(playlistId, cv.getAsString(PlaylistSongs.PLAYLIST_ID));
		assertEquals(songId, cv.getAsString(PlaylistSongs.SONG_ID));
	}
	
	public void testFromGenre() {
		final String genre = "Drum'n'Base";
		final String genreId = MusicContentValues.getGenreId(genre);
		
		final ContentValues cv = MusicContentValues.forGenre(genre);
		
		assertEquals(genreId, cv.getAsString(Genres.GENRE_ID));
		assertEquals(genre, cv.getAsString(Genres.GENRE_TITLE));
	}
}
