package com.ubuntuone.android.music.provider;

import android.net.Uri;
import android.test.mock.MockContentResolver;

import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;

public final class MusicProviderBasicTest extends MusicProviderTestBase
{
	private static final Uri INVALID_URI =
			Uri.withAppendedPath(MusicContract.BASE_CONTENT_URI, "invalid");
	
	private MockContentResolver mMockResolver;

	public MusicProviderBasicTest() {
		super(MusicProvider.class, MusicContract.CONTENT_AUTHORITY);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		mMockResolver = getMockContentResolver();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}
	
	public void testInvalidUri() {
		String mimeType = mMockResolver.getType(INVALID_URI);
		assertNull(mimeType);
	}
	
	public void testArtistsUriAndGetType() {
		String mimeType = mMockResolver.getType(Artists.CONTENT_URI);
		assertEquals(Artists.CONTENT_TYPE, mimeType);
		
		Uri artistIdUri = Artists.buildArtistUri("artist-id");
		mimeType = mMockResolver.getType(artistIdUri);
		assertEquals(Artists.CONTENT_ITEM_TYPE, mimeType);
		
		Uri artistAlbumsUri = Artists.buildAlbumsUri("artist-id");
		mimeType = mMockResolver.getType(artistAlbumsUri);
		assertEquals(Albums.CONTENT_TYPE, mimeType);
	}
	
	public void testAlbumsUriAndGetType() {
		String mimeType = mMockResolver.getType(Albums.CONTENT_URI);
		assertEquals(Albums.CONTENT_TYPE, mimeType);
		
		Uri albumIdUri = Albums.buildAlbumUri("album-id");
		mimeType = mMockResolver.getType(albumIdUri);
		assertEquals(Albums.CONTENT_ITEM_TYPE, mimeType);
		
		Uri albumSongsUri = Albums.buildSongsUri("album-id");
		mimeType = mMockResolver.getType(albumSongsUri);
		assertEquals(Songs.CONTENT_TYPE, mimeType);
	}
	
	public void testSongsUriAndGetType() {
		String mimeType = mMockResolver.getType(Songs.CONTENT_URI);
		assertEquals(Songs.CONTENT_TYPE, mimeType);
		
		Uri songIdUri = Songs.buildSongUri("song-id");
		mimeType = mMockResolver.getType(songIdUri);
		assertEquals(Songs.CONTENT_ITEM_TYPE, mimeType);
	}
	
	public void testPlaylistsUriAndGetType() {
		String mimeType = mMockResolver.getType(Playlists.CONTENT_URI);
		assertEquals(Playlists.CONTENT_TYPE, mimeType);
		
		Uri playlistIdUri = Playlists.buildPlaylistUri("playlist-id");
		mimeType = mMockResolver.getType(playlistIdUri);
		assertEquals(Playlists.CONTENT_ITEM_TYPE, mimeType);
		
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri("playlist-id");
		mimeType = mMockResolver.getType(playlistSongsUri);
		assertEquals(Songs.CONTENT_TYPE, mimeType);
	}
}
