package com.ubuntuone.android.music.provider;

import java.util.ArrayList;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;

import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.Genres;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.provider.MusicContract.SongsColumns;
import com.ubuntuone.android.music.provider.dao.AlbumDao;
import com.ubuntuone.android.music.provider.dao.ArtistDao;
import com.ubuntuone.android.music.provider.dao.GenreDao;
import com.ubuntuone.android.music.provider.dao.PlaylistDao;
import com.ubuntuone.android.music.provider.dao.PlaylistSongDao;
import com.ubuntuone.android.music.provider.dao.SongDao;
import com.ubuntuone.android.music.util.HashUtils;
import com.ubuntuone.android.music.util.Lists;
import com.ubuntuone.api.music.model.U1Album;
import com.ubuntuone.api.music.model.U1Artist;
import com.ubuntuone.api.music.model.U1Playlist;
import com.ubuntuone.api.music.model.U1Song;

public class MusicProviderInsertTest extends MusicProviderMockDataTestBase {

	public MusicProviderInsertTest() {
		super(MusicProvider.class, MusicContract.CONTENT_AUTHORITY);
	}
	
	public void testArtistsInsert() {
		ArrayList<String> albumList = Lists.newArrayList();
		albumList.add("album5-album_id");
		albumList.add("album6-album_id");
		U1Artist artist = new U1Artist("artist3-artist_id", "artist3-artist_url",
				"artist3-artist_name", "artist3-artist_art_url", albumList, 42);
		ContentValues cv = MusicContentValues.forArtist(artist);
		
		Uri uri = mResolver.insert(Artists.CONTENT_URI, cv);
		
		assertNotNull("Inserted artist uri is null.", uri);
		
		Cursor cursor = mResolver.query(uri, Artists.DEFAULT_PROJECTION, null, null, null);
		assertEquals("Wrong artist row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong artist id.", artist.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_ID)));
		assertEquals("Wrong artist name.", artist.getArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_NAME)));
		assertEquals("Wrong artist album.", MusicContentValues.getAlbumList(artist),
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ALBUM_LIST)));
		assertEquals("Wrong artist song count.", (int) artist.getSongCount(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Artists.SONG_COUNT)));
		
		cursor.close();
	}
	
	public void testArtistDaoInsert() {
		ArrayList<String> albumList = Lists.newArrayList();
		albumList.add("album5-album_id");
		albumList.add("album6-album_id");
		U1Artist artist = new U1Artist("artist3-artist_id", "artist3-artist_url",
				"artist3-artist_name", "artist3-artist_art_url", albumList, 42);
		ArtistDao artistDao = new ArtistDao();
		SQLiteDatabase db = new MusicDatabase(getMockContext()).getWritableDatabase();
		artistDao.updateOrInsert(db, artist);
		db.close();
		
		Uri uri = Artists.buildArtistUri(artist.getId());
		
		Cursor cursor = mResolver.query(uri, Artists.DEFAULT_PROJECTION, null, null, null);
		assertEquals("Wrong artist row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());

		assertEquals("Wrong artist id.", artist.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_ID)));
		assertEquals("Wrong artist name.", artist.getArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_NAME)));
		assertEquals("Wrong artist album.", MusicContentValues.getAlbumList(artist),
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ALBUM_LIST)));
		assertEquals("Wrong artist song count.", (int) artist.getSongCount(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Artists.SONG_COUNT)));
		
		cursor.close();
	}
	
	public void testAlbumsInsert() {
		U1Album album = new U1Album("album5-album_id", "album5-album_url", "artist1-artist_name",
				"artist1-artist_id", "album5-album_title", 1345345345L, "album5-art-url", 2012);
		ContentValues cv = MusicContentValues.forAlbum(album);
		
		Uri uri = mResolver.insert(Albums.CONTENT_URI, cv);
		
		assertNotNull("Inserted album uri is null.", uri);
		
		Cursor cursor = mResolver.query(uri, Albums.DEFAULT_PROJECTION, null, null, null);
		assertEquals("Wrong album row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong album id.", album.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ID)));
		assertEquals("Wrong album title.", album.getTitle(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		assertEquals("Wrong album artist name.", album.getArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST)));
		assertEquals("Wrong album artist id.", album.getArtistId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST_ID)));
		assertEquals("Wrong album year.", (int) album.getYear(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Albums.ALBUM_YEAR)));
		assertEquals("Wrong album date added.", (long) album.getParsedDate(),
				cursor.getLong(cursor.getColumnIndexOrThrow(Albums.ALBUM_DATE_ADDED)));
		
		cursor.close();
	}
	
	public void testAlbumDaoInsert() {
		U1Album album = new U1Album("album5-album_id", "album5-album_url", "artist1-artist_name",
				"artist1-artist_id", "album5-album_title", 1345345345L, "album5-art-url", 2012);
		AlbumDao albumDao = new AlbumDao();
		SQLiteDatabase db = new MusicDatabase(getMockContext()).getWritableDatabase();
		albumDao.updateOrInsert(db, album);
		db.close();
		
		Uri uri = Albums.buildAlbumUri(album.getId());
		
		Cursor cursor = mResolver.query(uri, Albums.DEFAULT_PROJECTION, null, null, null);
		assertEquals("Wrong album row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong album id.", album.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ID)));
		assertEquals("Wrong album title.", album.getTitle(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		assertEquals("Wrong album artist name.", album.getArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST)));
		assertEquals("Wrong album artist id.", album.getArtistId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST_ID)));
		assertEquals("Wrong album year.", (int) album.getYear(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Albums.ALBUM_YEAR)));
		assertEquals("Wrong album date added.", (long) album.getParsedDate(),
				cursor.getLong(cursor.getColumnIndexOrThrow(Albums.ALBUM_DATE_ADDED)));
		
		cursor.close();
	}
	
	public void testSongsInsert() {
		U1Song song = new U1Song("song5-song_id", "song5-song_url",
				HashUtils.md5("album1-album_titleartist1-artist_name"), "song5-album_artist",
				"album1-album_title", "song5-song_art_url", "genre2", 2009, 3, 2, 315,
				1024 * 1024 * 2, 256, "artist1-artist_name", HashUtils.md5("artist1-artist_name"),
				"song5-song_title", "mp3", "audio/mpeg", "song5-song_stream", null);
		ContentValues cv = MusicContentValues.forSong(song);
		
		Uri uri = mResolver.insert(Songs.CONTENT_URI, cv);
		
		assertNotNull("Inserted song uri is null.", uri);
		
		Cursor cursor = mResolver.query(uri, Songs.getDefaultProjection(), null, null, null);
		assertEquals("Wrong song row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong song id.", song.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ID)));
		assertEquals("Wrong song title.", song.getTitle(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
		assertEquals("Wrong song album.", song.getAlbum(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM)));
		assertEquals("Wrong song album id.", song.getAlbumId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM_ID)));
		assertEquals("Wrong song album artist.", song.getAlbumArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM_ARTIST)));
		assertEquals("Wrong song artist.", song.getArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ARTIST)));
		assertEquals("Wrong song artist id.", song.getArtistId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ARTIST_ID)));
		assertEquals("Wrong song genre.", song.getGenre(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_GENRE)));
		assertEquals("Wrong song genre ids.", MusicContentValues.getGenreId(song.getGenre()),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_GENRE_ID)));
		assertEquals("Wrong song year.", (int) song.getYear(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_YEAR)));
		assertEquals("Wrong song disc number.", (int) song.getDiscNumber(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_DISC_NUMBER)));
		assertEquals("Wrong song track.", (int) song.getTrack(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_TRACK)));
		assertEquals("Wrong song duration.", (int) song.getDuration(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_DURATION)));
		assertEquals("Wrong song size.", (int) song.getSize(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_SIZE)));
		assertEquals("Wrong song bit rate.", (int) song.getBitRate(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_BIT_RATE)));
		assertEquals("Wrong song suffix.", song.getSuffix(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_SUFFIX)));
		assertEquals("Wrong song content type.", song.getContentType(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_CONTENT_TYPE)));
		assertEquals("Wrong song last played at.", 0L,
				cursor.getLong(cursor.getColumnIndexOrThrow(Songs.SONG_LAST_PLAYED_AT)));
		
		cursor.close();
	}
	
	public void testSongDaoInsert() {
		U1Song song = new U1Song("song5-song_id", "song5-song_url",
				HashUtils.md5("album1-album_titleartist1-artist_name"), "song5-album_artist",
				"album1-album_title", "song5-song_art_url", "genre2", 2009, 3, 2, 315,
				1024 * 1024 * 2, 256, "artist1-artist_name", HashUtils.md5("artist1-artist_name"),
				"song5-song_title", "mp3", "audio/mpeg", "song5-song_stream", null);
		SongDao songDao = new SongDao();
		SQLiteDatabase db = new MusicDatabase(getMockContext()).getWritableDatabase();
		songDao.updateOrInsert(db, song);
		db.close();
		
		Uri uri = Songs.buildSongUri(song.getId());
		
		Cursor cursor = mResolver.query(uri, Songs.getDefaultProjection(), null, null, null);
		assertEquals("Wrong song row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong song id.", song.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ID)));
		assertEquals("Wrong song title.", song.getTitle(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
		assertEquals("Wrong song album.", song.getAlbum(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM)));
		assertEquals("Wrong song album id.", song.getAlbumId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM_ID)));
		assertEquals("Wrong song album artist.", song.getAlbumArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM_ARTIST)));
		assertEquals("Wrong song artist.", song.getArtist(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ARTIST)));
		assertEquals("Wrong song artist id.", song.getArtistId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ARTIST_ID)));
		assertEquals("Wrong song genre.", song.getGenre(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_GENRE)));
		assertEquals("Wrong song genre ids.", MusicContentValues.getGenreId(song.getGenre()),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_GENRE_ID)));
		assertEquals("Wrong song year.", (int) song.getYear(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_YEAR)));
		assertEquals("Wrong song disc number.", (int) song.getDiscNumber(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_DISC_NUMBER)));
		assertEquals("Wrong song track.", (int) song.getTrack(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_TRACK)));
		assertEquals("Wrong song duration.", (int) song.getDuration(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_DURATION)));
		assertEquals("Wrong song size.", (int) song.getSize(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_SIZE)));
		assertEquals("Wrong song bit rate.", (int) song.getBitRate(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_BIT_RATE)));
		assertEquals("Wrong song suffix.", song.getSuffix(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_SUFFIX)));
		assertEquals("Wrong song content type.", song.getContentType(),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_CONTENT_TYPE)));
		assertEquals("Wrong song last played at.", 0L,
				cursor.getLong(cursor.getColumnIndexOrThrow(Songs.SONG_LAST_PLAYED_AT)));
		
		cursor.close();
	}
	
	public void testPlaylistsInsert() {
		U1Playlist playlist = new U1Playlist("playlist3-playlist_id", "playlist3-playlist_url",
				"playlist3-playlist_name", 10);
		ContentValues cv = MusicContentValues.forPlaylist(playlist);
		
		Uri uri = mResolver.insert(Playlists.CONTENT_URI, cv);
		
		assertNotNull("Inserted playlist uri is null.", uri);
		
		Cursor cursor = mResolver.query(uri, Playlists.DEFAULT_PROJECTION, null, null, null);
		assertEquals("Wrong playlist row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong playlist id.", playlist.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_ID)));
		assertEquals("Wrong playlist name.", playlist.getName(),
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_NAME)));
		assertEquals("Wrong playlist song count.", (int) playlist.getSongCount(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_SONG_COUNT)));
		
		cursor.close();
	}
	
	public void testPlaylistDaoInsert() {
		U1Playlist playlist = new U1Playlist("playlist3-playlist_id", "playlist3-playlist_url",
				"playlist3-playlist_name", 10);
		PlaylistDao playlistDao = new PlaylistDao();
		SQLiteDatabase db = new MusicDatabase(getMockContext()).getWritableDatabase();
		playlistDao.updateOrInsert(db, playlist);
		db.close();
		
		Uri uri = Playlists.buildPlaylistUri(playlist.getId());
		
		Cursor cursor = mResolver.query(uri, Playlists.DEFAULT_PROJECTION, null, null, null);
		assertEquals("Wrong playlist row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong playlist id.", playlist.getId(),
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_ID)));
		assertEquals("Wrong playlist name.", playlist.getName(),
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_NAME)));
		assertEquals("Wrong playlist song count.", (int) playlist.getSongCount(),
				cursor.getInt(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_SONG_COUNT)));
		
		cursor.close();
	}
	
	public void testPlaylistSongsInsert() {
		final String playlistId = "playlist1-playlist_id";
		final String songId = "song3-song_id";
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(playlistId);
		
		{ // Check preconditions.
			Cursor cursor = mResolver.query(playlistSongsUri, PlaylistSongs.DEFAULT_PROJECTION,
					null, null, PlaylistSongs.DEFAULT_SORT);
			assertEquals("Wrong playlist song row count returned.", 2, cursor.getCount());
			cursor.close();
		}
		
		// Insert new song.
		ContentValues cv = MusicContentValues.forPlaylistSong(playlistId, songId);
		Uri uri = mResolver.insert(playlistSongsUri, cv);
		
		assertNotNull("Inserted playlist song uri is null.", uri);
		
		Cursor cursor = mResolver.query(playlistSongsUri, PlaylistSongs.DEFAULT_PROJECTION,
				null, null, PlaylistSongs.DEFAULT_SORT);
		assertEquals("Wrong playlist song row count returned.", 3, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToPosition(2)); // move to third song
		
		assertEquals("Wrong playlist id.", playlistId,
				cursor.getString(cursor.getColumnIndexOrThrow(PlaylistSongs.PLAYLIST_ID)));
		assertEquals("Wrong playlist song id.", songId,
				cursor.getString(cursor.getColumnIndexOrThrow(SongsColumns.SONG_ID)));
		
		cursor.close();
	}
	
	public void testPlaylistSongDaoInsert() {
		final String playlistId = "playlist1-playlist_id";
		final String songId = "song3-song_id";
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri(playlistId);
		
		{ // Check preconditions.
			Cursor cursor = mResolver.query(playlistSongsUri, PlaylistSongs.DEFAULT_PROJECTION,
					null, null, PlaylistSongs.DEFAULT_SORT);
			assertEquals("Wrong playlist song row count returned.", 2, cursor.getCount());
			cursor.close();
		}
		
		// Insert new song.
		PlaylistSongDao playlistSongDao = new PlaylistSongDao();
		SQLiteDatabase db = new MusicDatabase(getMockContext()).getWritableDatabase();
		playlistSongDao.updateOrInsert(db, playlistId, songId);
		db.close();
		
		Cursor cursor = mResolver.query(playlistSongsUri, PlaylistSongs.DEFAULT_PROJECTION,
				null, null, PlaylistSongs.DEFAULT_SORT);
		assertEquals("Wrong playlist song row count returned.", 3, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToPosition(2)); // move to third song
		
		assertEquals("Wrong playlist id.", playlistId,
				cursor.getString(cursor.getColumnIndexOrThrow(PlaylistSongs.PLAYLIST_ID)));
		assertEquals("Wrong playlist song id.", songId,
				cursor.getString(cursor.getColumnIndexOrThrow(SongsColumns.SONG_ID)));
		
		cursor.close();
	}
	
	public void testGenresInsert() {
		// These genres are treated as same: md5(genre.toUpperCase())
		String genre1 = "acid jazz";
		String genre2 = "Acid JAZZ";
		
		ContentValues cv1 = MusicContentValues.forGenre(genre1);
		Uri uri = mResolver.insert(Genres.CONTENT_URI, cv1);
		assertNotNull("Inserted genre uri is null.", uri);
		
		ContentValues cv2 = MusicContentValues.forGenre(genre2);
		Uri uri2 = mResolver.insert(Genres.CONTENT_URI, cv2);
		assertNotNull("Inserted genre uri is null.", uri2);
		
		Cursor cursor = mResolver.query(Genres.CONTENT_URI,
				Genres.DEFAULT_PROJECTION, null, null, null);
				
		assertEquals("Wrong genre row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong genre name.", genre1,
				cursor.getString(cursor.getColumnIndexOrThrow(Genres.GENRE_TITLE)));
	}
	
	public void testGenreDaoInsert() {
		// These genres are treated as same: md5(genre.toUpperCase())
		String genre1 = "acid jazz";
		String genre2 = "Acid JAZZ";
		
		GenreDao genreDao = new GenreDao();
		SQLiteDatabase db = new MusicDatabase(getMockContext()).getWritableDatabase();
		genreDao.insert(db, genre1);
		genreDao.insert(db, genre2);
		db.close();
		
		Cursor cursor = mResolver.query(Genres.CONTENT_URI,
				Genres.DEFAULT_PROJECTION, null, null, null);
				
		assertEquals("Wrong genre row count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong genre name.", genre1,
				cursor.getString(cursor.getColumnIndexOrThrow(Genres.GENRE_TITLE)));
	}
}
