package com.ubuntuone.android.music.provider;

import android.database.Cursor;
import android.net.Uri;

import com.ubuntuone.android.music.MockData;
import com.ubuntuone.android.music.provider.MusicContract.Albums;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicContract.PlaylistSongs;
import com.ubuntuone.android.music.provider.MusicContract.Playlists;
import com.ubuntuone.android.music.provider.MusicContract.Songs;
import com.ubuntuone.android.music.util.HashUtils;

/*
 * <p>testQueryEntities - tests for number of rows and sort order
 * <p>testQueryEntitiesId - tests for number of rows and selection of column values
 * <p>testQueryEntitySubentities - tests for number of rows and sort order
 */
public class MusicProviderQueryTest extends MusicProviderMockDataTestBase
{
	public MusicProviderQueryTest() {
		super(MusicProvider.class, MusicContract.CONTENT_AUTHORITY);
	}
	
	public void testQueryArtists() {
		Cursor cursor = mResolver.query(Artists.CONTENT_URI,
				Artists.DEFAULT_PROJECTION, null, null, Artists.DEFAULT_SORT);
		assertEquals("Wrong artist count returned.", MockData.ARTISTS_COUNT, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Artist out of order.", "artist1-artist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_NAME)));
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Artist out of order.", "artist2-artist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_NAME)));
		
		cursor.close();
	}
	
	public void testQueryArtistsId() {		
		Uri artistUri = Artists.buildArtistUri(HashUtils.md5("artist1-artist_name"));
		Cursor cursor = mResolver.query(artistUri,
				Artists.DEFAULT_PROJECTION, null, null, null);
		
		assertEquals("Wrong artists count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong artist id.", HashUtils.md5("artist1-artist_name"),
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_ID)));
		assertEquals("Wrong artist name.", "artist1-artist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ARTIST_NAME)));
		assertEquals("Wrong artist album list.", "album1-album_id, album2-album_id",
				cursor.getString(cursor.getColumnIndexOrThrow(Artists.ALBUM_LIST)));
		assertEquals("Wrong artist song count.", 2,
				cursor.getInt(cursor.getColumnIndexOrThrow(Artists.SONG_COUNT)));
		
		cursor.close();
	}
	
	public void testQueryArtistAlbums() {
		Uri artistAlbumsUri = Artists.buildAlbumsUri(HashUtils.md5("artist1-artist_name"));
		Cursor cursor = mResolver.query(artistAlbumsUri,
				Albums.DEFAULT_PROJECTION, null, null, Albums.DEFAULT_SORT);
		
		assertEquals("Wrong artist albums count returned.", 2, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
//		assertEquals("Wrong album returned.", HashUtils.md5("artist1-artist_name"), // XXX
//				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST_ID)));
		assertEquals("Album out of order.", "album1-album_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
//		assertEquals("Wrong album returned.", HashUtils.md5("artist1-artist_name"), // XXX
//				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST_ID)));
		assertEquals("Album out of order.", "album2-album_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		
		cursor.close();
	}
	
	public void testQueryArtistSongs() {
		Uri artistSongsUri = Artists.buildSongsUri(HashUtils.md5("artist1-artist_name"));
		Cursor cursor = mResolver.query(artistSongsUri,
				Songs.getDefaultProjection(), null, null, Songs.DEFAULT_SORT);
		
		assertEquals("Wrong artist songs count returned.", 2, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Artist song out of order.", "song1-song_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Artist song out of order.", "song2-song_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
	}
	
	public void testQueryAlbums() {
		Cursor cursor = mResolver.query(Albums.CONTENT_URI,
				Albums.DEFAULT_PROJECTION, null, null, Albums.DEFAULT_SORT);
		
		assertEquals("Wrong album count returned.", MockData.ALBUMS_COUNT, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Album out of order.", "album1-album_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Album out of order.", "album2-album_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Album out of order.", "album21-album_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		
		cursor.close();
	}
	
	public void testQueryAlbumsId() {
		Uri albumUri = Albums.buildAlbumUri(
				HashUtils.md5("album1-album_titleartist1-artist_name"));
		Cursor cursor = mResolver.query(albumUri, Albums.DEFAULT_PROJECTION, null, null, null);
		
		assertEquals("Wrong albums count returned.", 1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong album id.", HashUtils.md5("album1-album_titleartist1-artist_name"),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ID)));
		assertEquals("Wrong album title.", "album1-album_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_TITLE)));
		assertEquals("Wrong album artist name.", "artist1-artist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST)));
		assertEquals("Wrong album artist id.", HashUtils.md5("artist1-artist_name"),
				cursor.getString(cursor.getColumnIndexOrThrow(Albums.ALBUM_ARTIST_ID)));
		assertEquals("Wrong album year.", 2001,
				cursor.getInt(cursor.getColumnIndexOrThrow(Albums.ALBUM_YEAR)));
		assertEquals("Wrong album date added.", 1300300300L,
				cursor.getLong(cursor.getColumnIndexOrThrow(Albums.ALBUM_DATE_ADDED)));
		
		cursor.close();
	}
	
	public void testQueryAlbumSongs() {
		Uri albumSongsUri = Albums.buildSongsUri(
				HashUtils.md5("album1-album_titleartist1-artist_name"));
		Cursor cursor = mResolver.query(albumSongsUri,
				Songs.getDefaultProjection(), null, null, Songs.DEFAULT_SORT);
		
		assertEquals("Wrong album songs count returned.", 2, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Album song out of order.", "song1-song_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Album song out of order.", "song2-song_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
	}
	
	public void testQuerySongs() {
		Cursor cursor = mResolver.query(Songs.CONTENT_URI,
				Songs.getDefaultProjection(), null, null, Songs.DEFAULT_SORT);
		
		assertEquals("Wrong song count returned.", MockData.SONGS_COUNT, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Song out of order.", "song1-song_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Song out of order.", "song2-song_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
		
		cursor.close();
	}
	
	public void testQuerySongsId() {
		Uri songUri = Songs.buildSongUri("song1-song_id");
		Cursor cursor = mResolver.query(songUri,
				Songs.getDefaultProjection(), null, null, null);
		
		assertEquals("Wrong songs count returned.",
				1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
				
		assertEquals("Wrong song id.", "song1-song_id",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ID)));
		assertEquals("Wrong song title.", "song1-song_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_TITLE)));
		assertEquals("Wrong song album.", "album1-album_title",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM)));
		assertEquals("Wrong song album id.",
				HashUtils.md5("album1-album_titleartist1-artist_name"),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM_ID)));
		assertEquals("Wrong song album artist.", "song1-song_album_artist",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ALBUM_ARTIST)));
		assertEquals("Wrong song artist.", "artist1-artist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ARTIST)));
		assertEquals("Wrong song artist id.", HashUtils.md5("artist1-artist_name"),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ARTIST_ID)));
		assertEquals("Wrong song genre.", "genre1",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_GENRE)));
		assertEquals("Wrong song genre ids.", HashUtils.md5("genre1"),
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_GENRE_ID)));
		assertEquals("Wrong song year.", 2001,
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_YEAR)));
		assertEquals("Wrong song disc number.", 1,
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_DISC_NUMBER)));
		assertEquals("Wrong song track.", 1,
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_TRACK)));
		assertEquals("Wrong song duration.", 180,
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_DURATION)));
		assertEquals("Wrong song size.", 1024 * 1024 * 1,
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_SIZE)));
		assertEquals("Wrong song bit rate.", 128 * 1,
				cursor.getInt(cursor.getColumnIndexOrThrow(Songs.SONG_BIT_RATE)));
		assertEquals("Wrong song suffix.", "mp3",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_SUFFIX)));
		assertEquals("Wrong song content type.", "audio/mpeg",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_CONTENT_TYPE)));
		assertEquals("Wrong song path.", null,
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_PATH)));
		assertEquals("Wrong song last played at.", 1347324619016L,
				cursor.getLong(cursor.getColumnIndexOrThrow(Songs.SONG_LAST_PLAYED_AT)));
		
		cursor.close();
	}
	
	public void testQueryPlaylists() {
		Cursor cursor = mResolver.query(Playlists.CONTENT_URI,
				Playlists.DEFAULT_PROJECTION, null, null, Playlists.DEFAULT_SORT);
		assertEquals("Wrong playlist count returned.", MockData.PLAYLISTS_COUNT, cursor.getCount());
		
		// Skip the play queue playlist.
		assertTrue("Cursor missing rows.", cursor.moveToNext());

		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Playlist out of order.", "playlist1-playlist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_NAME)));
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		assertEquals("Playlist out of order.", "playlist2-playlist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_NAME)));
		
		cursor.close();
	}
	
	public void testQueryPlaylistsId() {
		Uri playlistUri = Playlists.buildPlaylistUri("playlist1-playlist_id");
		Cursor cursor = mResolver.query(playlistUri,
				Playlists.DEFAULT_PROJECTION, null, null, null);
		
		assertEquals("Wrong playlists count returned.",
				1, cursor.getCount());
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		
		assertEquals("Wrong playlist id.", "playlist1-playlist_id",
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_ID)));
		assertEquals("Wrong playlist name.", "playlist1-playlist_name",
				cursor.getString(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_NAME)));
		assertEquals("Wrong playlist song count.", 2,
				cursor.getInt(cursor.getColumnIndexOrThrow(Playlists.PLAYLIST_SONG_COUNT)));
		
		cursor.close();
	}
	
	public void testQueryPlaylistSongs() {
		Uri playlistSongsUri = Playlists.buildPlaylistSongsUri("playlist1-playlist_id");
		Cursor cursor = mResolver.query(playlistSongsUri,
				PlaylistSongs.DEFAULT_PROJECTION, null, null, PlaylistSongs.DEFAULT_SORT);
		assertEquals("Wrong playlist songs count returned.", 2, cursor.getCount());
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		int song1_id = cursor.getInt(cursor.getColumnIndexOrThrow(Songs._ID));
		assertEquals("Wrong song returned.", "song2-song_id",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ID)));
		
		assertTrue("Cursor missing rows.", cursor.moveToNext());
		int song2_id = cursor.getInt(cursor.getColumnIndexOrThrow(Songs._ID));
		assertEquals("Wrong song returned.", "song1-song_id",
				cursor.getString(cursor.getColumnIndexOrThrow(Songs.SONG_ID)));
		
		assertTrue("Playlist songs out of order.", song1_id < song2_id);
		
		cursor.close();
	}
	
	public void _testQueryGenres() {
		fail("Not implemented yet.");
	}
	
	public void _testQueryGenreSongs() {
		fail("Not implemented yet.");
	}
}
