package com.ubuntuone.android.music.ui;
import android.app.ActionBar;
import android.app.Activity;
import android.app.Instrumentation;
import android.app.Instrumentation.ActivityMonitor;
import android.database.sqlite.SQLiteDatabase;
import android.support.v4.view.ViewPager;
import android.test.ActivityInstrumentationTestCase2;
import android.view.KeyEvent;
import android.widget.ListView;

import com.ubuntuone.android.music.MockData;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicContract.Artists;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicProviderUtils;
import com.ubuntuone.android.test.UIUtils;

public class ArtistsPageTest extends ActivityInstrumentationTestCase2<HomeActivity>
{
	private Instrumentation mInstrumentation;
	private ActivityMonitor mArtistActivityMonitor;
	private ActivityMonitor mPlayerActivityMonitor;

	private SQLiteDatabase mDatabase;
	private HomeActivity mActivity;
	private ActionBar mActionBar;
	private ListView mList;

	public ArtistsPageTest() {
		super(HomeActivity.class);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
		mInstrumentation = getInstrumentation();
		
		mDatabase = new MusicDatabase(getActivity()).getWritableDatabase();
		MockData.setUp(mDatabase);
		
		mPlayerActivityMonitor = new ActivityMonitor(
				PlayerActivity.class.getName(), null, false);
		mInstrumentation.addMonitor(mPlayerActivityMonitor);
		
		mArtistActivityMonitor = new ActivityMonitor(
				ArtistActivity.class.getName(), null, false);
		mInstrumentation.addMonitor(mArtistActivityMonitor);

		mActivity = getActivity();
		mActivity.onProgress(R.id.artist_loader_id); // Simulate finished sync.
		mActionBar = mActivity.getActionBar();
		
		mInstrumentation.waitForIdleSync();
	}

	@Override
	protected void tearDown() throws Exception {
		mInstrumentation.removeMonitor(mPlayerActivityMonitor);
		mInstrumentation.removeMonitor(mArtistActivityMonitor);
		
		if (!mActivity.isFinishing()) mActivity.finish();
		mActionBar = null;
		mActivity = null;
		
		MockData.tearDown(mDatabase);
		mDatabase.close();
		mDatabase = null;
		
		super.tearDown();
	}

	private void openArtistsTab() {
		UIUtils.openTab(mActionBar, 0);
		ViewPager pager = (ViewPager) mActivity.findViewById(R.id.pager);
		mList = (ListView) pager.findViewWithTag("artist_list");
	}
	
	public void testPlayQueueExists() {
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mActivity);
		assertTrue(MusicProviderUtils.QUEUE_NAME + " does not exist.", songCount != -1);
		
		mInstrumentation.waitForIdleSync();
	}
	
	public void testShowArtistActivityOnListItemClick() {
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				openArtistsTab();
				mList.requestFocus();
			}
		});
		
		getActivity().getContentResolver().notifyChange(Artists.CONTENT_URI, null);
		
		mInstrumentation.waitForIdleSync();
		
		// --- Click on the first artist ---
		sendKeys(KeyEvent.KEYCODE_DPAD_DOWN);
		mInstrumentation.sendKeyDownUpSync(KeyEvent.KEYCODE_DPAD_CENTER);
		
		// {@link ArtistActivity} should have launched.
		Activity activity = mArtistActivityMonitor.waitForActivityWithTimeout(5 * 1000);
		assertTrue("ArtistActivity was not started.",
				mInstrumentation.checkMonitorHit(mArtistActivityMonitor, 1));
		activity.finish();
		
		mInstrumentation.waitForIdleSync();
	}
}
