package com.ubuntuone.android.music.ui;
import android.app.Activity;
import android.app.Instrumentation;
import android.app.Instrumentation.ActivityMonitor;
import android.content.pm.ActivityInfo;
import android.database.sqlite.SQLiteDatabase;
import android.support.v4.view.ViewPager;
import android.test.ActivityInstrumentationTestCase2;
import android.view.View;

import com.actionbarsherlock.internal.view.menu.MenuBuilder;
import com.actionbarsherlock.view.MenuItem;
import com.ubuntuone.android.music.MockData;
import com.ubuntuone.android.music.R;
import com.ubuntuone.android.music.provider.MusicDatabase;
import com.ubuntuone.android.music.provider.MusicProviderUtils;

public class HomeActivityTest extends ActivityInstrumentationTestCase2<HomeActivity>
{
	private Instrumentation mInstrumentation;
	private ActivityMonitor mSettingsActivityMonitor;
	private ActivityMonitor mPlayerActivityMonitor;
	
	private SQLiteDatabase mDatabase;
	private HomeActivity mActivity;
	
	public HomeActivityTest() {
		super(HomeActivity.class);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
		mInstrumentation = getInstrumentation();
		
		mDatabase = new MusicDatabase(getActivity()).getWritableDatabase();
		MockData.setUp(mDatabase);
		
		mSettingsActivityMonitor = new ActivityMonitor(
				SettingsActivity.class.getName(), null, false);
		mInstrumentation.addMonitor(mSettingsActivityMonitor);
		
		mPlayerActivityMonitor = new ActivityMonitor(
				PlayerActivity.class.getName(), null, false);
		mInstrumentation.addMonitor(mPlayerActivityMonitor);
		
		mActivity = getActivity();
		
		mInstrumentation.waitForIdleSync();
	}

	@Override
	protected void tearDown() throws Exception {
		mInstrumentation.removeMonitor(mSettingsActivityMonitor);
		mInstrumentation.removeMonitor(mPlayerActivityMonitor);
		
		if (!mActivity.isFinishing()) mActivity.finish();
		mActivity = null;
		
		MockData.tearDown(mDatabase);
		mDatabase.close();
		mDatabase = null;
		
		super.tearDown();
	}
	
	public void testHasViewPager() {
		ViewPager pager = (ViewPager) mActivity.findViewById(R.id.pager);
		assertNotNull("View pager missing.", pager);
		assertTrue("View pager not visible.", pager.getVisibility() == View.VISIBLE);
	}
	
	public void testViewPagerHasFiveTabs() {
		ViewPager pager = (ViewPager) mActivity.findViewById(R.id.pager);
		int pageCount = pager.getAdapter().getCount();
		assertEquals("Wrong view pager page count.", 5, pageCount);
	}
	
	private MenuBuilder getOptionsMenu() {
		MenuBuilder builder = new MenuBuilder(mInstrumentation.getTargetContext());
		mActivity.onCreateOptionsMenu(builder);
		mActivity.onPrepareOptionsMenu(builder);
		return builder;
	}
	
	public void testOptionsMenuContainsRefresh() {
		MenuItem item = (MenuItem) getOptionsMenu().findItem(R.id.menu_refresh);
		assertTrue("Refresh options menu not visible or not enabled.",
				item.isVisible() && item.isEnabled());
		mActivity.closeOptionsMenu();
	}
	
	public void testOptionsMenuContainsShuffleAll() {
		MenuItem item = (MenuItem) getOptionsMenu().findItem(R.id.menu_shuffle_all);
		assertTrue("Shuffle all options menu not visible or not enabled.",
				item.isVisible() && item.isEnabled());
		mActivity.closeOptionsMenu();
	}
	
	public void testOptionsMenuShuffleAll() {
		int songCount = MusicProviderUtils.getPlayQueueSongCount(mActivity);
		assertTrue(MusicProviderUtils.QUEUE_NAME + " does not exist.", songCount != -1);
		
		assertFalse("Shuffle all songs already launched.", mActivity._shuffleAllSongsTaskLaunched);
		
		mInstrumentation.invokeMenuActionSync(mActivity, R.id.menu_shuffle_all, 0);
		assertTrue("Shuffle all songs task not launched.", mActivity._shuffleAllSongsTaskLaunched);
	}
	
	public void testOptionsMenuContainsSettings() {
		MenuItem item = (MenuItem) getOptionsMenu().findItem(R.id.menu_settings);
		assertTrue("Settings options menu not visible or not enabled.",
				item.isVisible() && item.isEnabled());
		mActivity.closeOptionsMenu();
	}
	
	public void testOptionsMenuSettings() {
		mInstrumentation.invokeMenuActionSync(mActivity, R.id.menu_settings, 0);
		
		Activity activity = mSettingsActivityMonitor.waitForActivityWithTimeout(5 * 1000);
		assertTrue("SettingsActivity was not started.",
				mInstrumentation.checkMonitorHit(mSettingsActivityMonitor, 1));
		activity.finish();
	}
	
	public void testOptionsMenuContainsHelp() {
		MenuItem item = (MenuItem) getOptionsMenu().findItem(R.id.menu_help);
		assertTrue("Help options menu not visible or not enabled.",
				item.isVisible() && item.isEnabled());
		mActivity.closeOptionsMenu();
	}
	
	// TODO Exercise Help options menu item.
	
	public void testOptionsMenuContainsSignOut() {
		MenuItem item = (MenuItem) getOptionsMenu().findItem(R.id.menu_sign_out);
		assertTrue("Sign out options menu not visible or not enabled.",
				item.isVisible() && item.isEnabled());
		mActivity.closeOptionsMenu();
	}
	
	// TODO Exercise Sign out options menu item.
	
	public void testNowPlayingClickInPortraitOrientationOpensPlayer() {
		mActivity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		mInstrumentation.waitForIdleSync();
		
		final View view = mActivity.findViewById(R.id.now_playing);
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				view.performClick();
			}
		});
		mInstrumentation.waitForIdleSync();
		
		Activity activity = mPlayerActivityMonitor.waitForActivityWithTimeout(5 * 1000);
		assertTrue("PlayerActivity was not started.",
				mInstrumentation.checkMonitorHit(mPlayerActivityMonitor, 1));
		activity.finish();
	}
	
	public void testNowPlayingClickInHorizontalOrientationOpensPlayer() {
		mActivity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		mInstrumentation.waitForIdleSync();
		
		final View view = mActivity.findViewById(R.id.now_playing);
		mActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				view.performClick();
			}
		});
		mInstrumentation.waitForIdleSync();
		
		Activity activity = mPlayerActivityMonitor.waitForActivityWithTimeout(5 * 1000);
		assertTrue("PlayerActivity was not started.",
				mInstrumentation.checkMonitorHit(mPlayerActivityMonitor, 1));
		activity.finish();
	}
}
